#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2008 Mario Arnaldos

#This file is part of HDLorean.
#
#HDLorean is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 2 of the License.

#HDLorean is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with HDLorean.  If not, see <http://www.gnu.org/licenses/>.


import os
import sys
import dbus
import calendar
import time
from optparse import OptionParser

#FIXME: if an installation is done, bring FrontendProxy to this module
from FrontendProxy import FrontendProxy
#import FrontendProxy


class hdlorean(object):
	
	__initiated=not os.path.exists(os.path.expanduser("~/.hdlorean/hdlorean.pid"))
	__frontend=None
	__abbrevs = [(1<<50, 'P'),(1<<40, 'T'),(1<<30, 'G'), (1<<20, 'M'), (1<<10, 'k'),(1, '')]


	##@brief Create a Snapshot, adding the path to the config file if not set before.
	#
	#@param pathList list of paths to backup
	#@return true if the backup set was succesfully made
	#@author Ender
	def makeSnapshot(self):
		print "Making snapshot"
		self.__frontend.snapshotRequest("USER")
	
	##@brief prints the list of snapshots in time order.
	#
	#@param storageMngr The StoreManager
	#@author Ender
	def listTimeOfSnapshots(self):
		print "Listing Snapshots"
		print "Still not implemented"
		#for timest in self.__frontend.getAllTimestamps():
		#	print timest
		#	print time.gmtime(float(timest))
	
	
	##@brief Lists the content of a path in a date of snapshot.
	#
	#@param timestamp Time of the snapshot of the info showed
	#@author Ender
	def getSnapshotInfo(self, timestamp):
		#print timestamp
		print "Geting snapshot info"
		dirList=self.__frontend.getCompleteDirList("/")
		print "Recoverable paths"
		for path in dirList:
			print path[0]
	
	##@brief Prints the history of a path
	#
	#@param path Path to search in time
	#@author Ender
	def getHistory(self, path):
		datos=self.__frontend.getHistorical(path)
		for backup in datos:
			size=self.__greek(backup[4])
			print "Date: "+backup[5][:19]+" Size: "+size+" Operation: "+backup[6]
	
	##@brief Change size to greek numeration
	#
	#@param size Size to convert
	def __greek(self, size):
		"""Return a string representing the greek/metric suffix of a size"""
		if size ==0:
			return "0.0 K"
		for factor, suffix in self.__abbrevs:
			if size > factor:
				return `int(size/factor)` + suffix
    
	##@brief Recover the previous snapshot of a path.
	#
	#@param destinationPath Path where files are to get placed
	#@param timestamp Timestamp of the snapshot to recover
	#@param pathList List of paths recovered
	#@return Path where files are recovered
	#@author Ender
	def recoverBackups(self, destinationPath, timestamp, pathList):
		print "Recovering Backup"
		
		#print destinationPath
		#print pathList
		#print timestamp
		
		if pathList==None:
			return self.__frontend.recoverBackups(timestamp,[''],destinationPath)
		else:
			return self.__frontend.recoverBackups(timestamp,pathList, destinationPath)
	
	
	##@brief Erase all versions of given path(s).
	#
	#@param paths paths to erase
	#@author Ender
	def eraseSnapshot(self,paths):
		print "Removing snapshot"
		self.__frontend.deleteAllVersions(paths)
			
	#@brief Remove some version(s) of given path(s)
	# 
	#
	#@author Ender
	def removePaths(self, path, timestamp):
		#TODO insert timestamps, and so
		self.__frontend.deleteTheseVersions(path,[timestamp])
		print "Removed "+ path
		
	##@brief Stop the hdlorean daemon.
	#
	#@author Ender
	def stopDaemon(self):
		self.__frontend.stopDaemon()
	
	#@brief Get a list of the paths, internal function, do NOT use this!!
	def __pathRecover(self,option, opt_str, value, parser):
		assert value is None
		value = []
		rargs = parser.rargs
		parse=True
		if len(rargs)<1:
			print "Some argument needed."
			value=False
			parse=False
			
		
		while rargs and parse:
			arg = rargs[0]
	
			# Stop if we hit an arg like "--foo", "-a", "-fx", "--file=f",
			# etc.
			if ((arg[:2] == "--" and len(arg) > 2) or
			(arg[:1] == "-" and len(arg) > 1 and arg[1] != "-")):
				break
			else:
				value.append(arg)
				del rargs[0]
		setattr(parser.values, option.dest, value)
		
	#@brief Parse the input timestamp. Internal function, do NOT use this!!
	def __makeTimestamp(self, option, opt_str, value, parser):
		assert value is None
		
		value=time.gmtime()
		
		#print value
		#arguments=True
		done=0
		rargs=parser.rargs
		if len(rargs)<1:
			print "More than 0 arguments needed. Default taken"
			done=6
			#arguments=False
		while rargs and done < 5:
			arg=rargs[0]
	
			# Stop if we hit an arg like "--foo", "-a", "-fx", "--file=f",
			# etc.
			if ((arg[:2] == "--" and len(arg) > 2) or
			(arg[:1] == "-" and len(arg) > 1 and arg[1] != "-")):
				break
			else:
				value[done]=int(rargs[0])
				del rargs[0]
			done=done+1
		
		value=calendar.timegm(value)

		setattr(parser.values, option.dest, value)

	##@brief Initiates the frontend proxy instance
	#
	#@author ender
	def initiate(self):
			self.__frontend=FrontendProxy()
	
	##@brief Main loop to catch the command line arguments and do the right things.
	#
	#@author Ender
	def main(self):
		#print "entrando"
		#Create the proxy
		
			
		#Create parser and populate it
		usate="Usage: %prog [options] args"
		parser=OptionParser()

		#FIXME originally used to force orverwriting, maybe not used in the future
		#parser.add_option("-f","--force", action="store_true", dest="force", default=False,help="Force writing")
		parser.add_option("-l","--list", action="store_true", dest="listar", default=False,
						help="Lists all the Snapshots of your system")
		parser.add_option("-r","--recover", action="store_true", dest="recover", default=False,
						help="Recover files. Default all in snapshot. If not time is given, last snapshot. If not backup path(r) is given, all files in snapshot.")
		parser.add_option("-b", "--backup=",
						action="callback",callback=self.__pathRecover,dest="pathList",default=None,
						help="Recover just some files. Recover the path(s) you define")
		parser.add_option("-T","--target", dest="destinationPath",default=os.getcwd(),
						help="Destination path for the recover action. If -T is not called, actual directory is used.")
		parser.add_option("-t","--time", action="callback",
		callback=self.__makeTimestamp,dest="timestamp",default=1,
		help="Set timestamp for consults or recoverings. If the option is not called, last snapshot is used. Time format: YYYY MM DD HH MM SS. Snapshot taken will be the one just before that moment")
		parser.add_option("-m","--make",	action="store_true", dest="makeSnapshot",default=False,
						help="Make a backup in the preferences defined place.")
		parser.add_option("-i","--info", action="store_true",dest="info",default=False,
						help="Get information about a given snapshot: files, state, etc. If not time is given with -t, last snapshot information retrieved")	
		parser.add_option("-H","--history", action="store",type="string", dest="historyPaths", default=False, 						help="Get the full time information of a given path")	
		#TODO changue erase format
		parser.add_option("-e","--erase",action="store",type="string",dest="erasePath",default=False,
						 help="Remove all backups from given path")
		parser.add_option("-d","--delete",action="store",type="string",dest="deletePath",default=False,
						help="Removes a backup of the file you write. The copy removed can be changed with -t option. Default=last.")
		parser.add_option("-s","--start",action="store_true",dest="start",default=False,
						help="Start the hdlorean daemon")
		parser.add_option("-k","--kill",action="store_true",dest="stop",default=False,
						help="Stop the hdlorean daemon")

		
		(options, args) = parser.parse_args()
		
		#FIXME: Quitar esto cuando se entregue
		#print parser.parse_args()
		
		printpath=""
		if options.listar:
			self.initiate()
			self.getHistory("/")
			
		elif options.recover:
			self.initiate()
			if options.timestamp==1:
				options.timestamp=long(self.__frontend.lastTimestamp())
	
			printpath=self.recoverBackups(options.destinationPath, options.timestamp, options.pathList)
			print "Recovered files in:"
			print printpath

		elif options.info:
			self.initiate()
			if options.timestamp==1:
				options.timestamp=long(self.__frontend.lastTimestamp())
			self.getSnapshotInfo(options.timestamp)

		elif options.makeSnapshot:
			self.initiate()
			self.makeSnapshot()

		elif options.historyPaths:
			self.initiate()
			self.getHistory(options.historyPaths)

		elif options.erasePath:
			self.initiate()
			self.eraseSnapshot(options.erasePath)

		elif options.deletePath:
			self.initiate()
			self.removePaths(options.deletePath,options.timestamp)		
		elif options.start:
			self.__initiated=False
			self.initiate()
			
		elif options.stop and not self.__initiated:
			self.initiate()
			print "Stoping HDLorean Daemon"
			self.stopDaemon()

		else:
			parser.print_help()
			
		if self.__initiated and self.__frontend:
			self.stopDaemon()
		
			
if __name__ == "__main__":
	console=hdlorean()
	console.main()
