# -*- coding: utf-8 -*-

#Copyright (C) 2008 Federico Mon Trotti, Daniel Tabas, Carlos Agra, Roberto Cano

#This file is part of HDLorean.
#
#HDLorean is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 2 of the License.

#HDLorean is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with HDLorean.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import subprocess
import shutil
import pygtk
import gtk
import gtk.glade
import gnome.ui
import gettext

##@brief Method that allows user to select a directory with a file chooser
# @author Carlos
def DirectorySelect(title='Please select a directory'):
	dialog = gtk.FileChooserDialog(title, None, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, 
		(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
	dialog.set_current_folder(os.environ['HOME'])
	
	if dialog.run() == gtk.RESPONSE_OK:
		directory = dialog.get_filename() + '/'
	else:
		directory = None

	dialog.destroy()
	return directory

## @brief Shows configuration Assistant
# @author Carlos
class Assistant:
	
	# List of indexed directories
	__dirsAdded = []
	# List of excluded directories
	__dirsExcluded = []
	__gladefile= "../../share/glade/HDLoreanAssistantMenu.glade"
	# Liststore & treeview for included directories
	__liststoreInclude = gtk.ListStore(str)
	__treeviewInclude = gtk.TreeView (__liststoreInclude)
	# Liststore & treeview for excluded directories
	__liststoreExclude = gtk.ListStore(str)
	__treeviewExclude = gtk.TreeView (__liststoreExclude)
	
	def __init__(self):
		APP='Assistant'
		DIR='../../share/locale/'
		gettext.bindtextdomain(APP, DIR)
		gettext.textdomain(APP)
		_ = gettext.gettext
		gtk.glade.bindtextdomain(APP, DIR)
		gtk.glade.textdomain(APP)
		# Load of glade file for the Assistant		
		self.wTree = gtk.glade.XML (self.__gladefile)		

		# Prepare the renderers
		textrenderer = gtk.CellRendererText()

		# Include initializations
		self.__treeviewInclude = self.wTree.get_widget('treeviewInclude')
		self.__treeviewInclude.set_model(self.__liststoreInclude)
		self.__treeviewInclude.set_headers_visible(True)
		# Add column 'Path' to the include treeview
		header, col, renderer = 'Path', 0, textrenderer
		column = gtk.TreeViewColumn(header, renderer, text=col)
		column.set_sort_column_id(col)
		column.set_resizable(True)
		self.__treeviewInclude.append_column(column)		
		# Exclude initializations
		self.__treeviewExclude = self.wTree.get_widget('treeviewExclude')
		self.__treeviewExclude.set_model(self.__liststoreExclude)
		self.__treeviewExclude.set_headers_visible(True)
		# Add column 'Path' to the exclude treeview
		header2, col2, renderer2 = 'Path', 0, textrenderer
		column2 = gtk.TreeViewColumn(header2, renderer2, text=col2)
		column2.set_sort_column_id(col2)
		column2.set_resizable(True)
		self.__treeviewExclude.append_column(column2)

		# dictionary: connects the signals with the Main Window functions
		dic = { "on_addButtonInclude_clicked" : self.addButtonIncludeClicked, 
			"on_removeButtonInclude_clicked" : self.removeButtonIncludeClicked,
			"on_addButtonExclude_clicked" : self.addButtonExcludeClicked,
			"on_removeButtonExclude_clicked" : self.removeButtonExcludeClicked,
			"on_assistantDruid_cancel" : self.cancelButtonClicked,
			"on_assistantDruid_help" : self.helpButtonClicked
		}		
		self.wTree.signal_autoconnect (dic)
		# TODO Load directories from config file

	def addButtonIncludeClicked(self,widget):
				
		directory = DirectorySelect('Please select a directory to back up')
		if not directory:
			return
		# TODO Check if directory was already indexed (config file)
		
		self.__dirsAdded.append(directory)
		#jarl = self.__dirsAdded.Check(directory)
		#print jarl
		itt = self.__liststoreInclude.append([directory])	
			
	def removeButtonIncludeClicked(self,widget):
		try:
			model,itt = self.__treeviewInclude.get_selection().get_selected()		# Returns a (TreeModel, TreeIter) tuple
			selection = self.__liststoreInclude.get_value(itt, 0)
			self.__liststoreInclude.remove(itt)
		except:
			print "meeeec"
			pass

	def addButtonExcludeClicked (self,widget):
		directory = DirectorySelect('Please select a directory to back up')
		if not directory:
			return

		# TODO Check if directory was already excluded (config file)

		self.__dirsExcluded.append(directory)
		itt = self.__liststoreExclude.append([directory])
		
	def removeButtonExcludeClicked (self,widget):
		try:
			model,itt = self.__treeviewExclude.get_selection().get_selected()		# Returns a (TreeModel, TreeIter) tuple
			selection = self.__liststoreExclude.get_value(itt, 0)
			self.__liststoreExclude.remove(itt)
		except:
			print "meeeec"
			pass

	def cancelButtonClicked(self,widget):
		gtk.main_quit()
	def helpButtonClicked(self,widget):
		print "Quien rie ahora?"	
	
# The call for Main Page
app=Assistant()
gtk.main()
