# -*- coding: utf-8 -*-

#Copyright (C) 2008 Federico Mon Trotti, Daniel Tabas, Carlos Agra, Roberto Cano

#This file is part of HDLorean.
#
#HDLorean is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#HDLorean is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with HDLorean.  If not, see <http://www.gnu.org/licenses/>.

import gobject
import dbus, dbus.glib, dbus.service
import sys
import os
import time

class FrontendProxy:

	# Needed to implement Singleton pattern
	__instance = False
	__server = None
	__serverWatcher = None
	__serverScheduler = None
	__serverSnapshot = None
	__backendRunning = False
	def __init__(self):
		bus = dbus.SessionBus()
		bus.add_signal_receiver(self.__handler,'signalStarted','com.hdlorean.Signals','com.hdlorean.Server','/hdlorean/signals')
		if not self.__instance:
			self.__instance = True
			# Daemon was succesfully initialized
			initialized = False
			# Times we try to start the daemon
			i = 0
			# Lists of PIDs of the started daemons
			pid = []
			#FIXME: Killing all the daemons except the first one is probably a bad idea. 05:02 am is not a good time for programming.
			while ((i < 5) and not initialized and not self.__backendRunning):
				try:
					
					self.__server = dbus.Interface(bus.get_object('com.hdlorean.Server', '/hdlorean'),'com.hdlorean.Interface')
					self.__serverWatcher = dbus.Interface(bus.get_object('com.hdlorean.Server', '/hdlorean'),'com.hdlorean.Watcher')
					self.__serverScheduler = dbus.Interface(bus.get_object('com.hdlorean.Server', '/hdlorean'),'com.hdlorean.Scheduler')
					self.__serverSnapshotManager = dbus.Interface(bus.get_object('com.hdlorean.Server', '/hdlorean'),'com.hdlorean.Snapshot')
					initialized = True
				except dbus.exceptions.DBusException, e:
					print "There isn't a hdlorean daemon running. Im gonna start one for you..."
					pid.append(os.spawnl(os.P_NOWAIT,"../hdloreand/hdloreand.py"))
					time.sleep(2)
					#self.__signalAwaiter()
				#finally:
				i = i + 1
			if not initialized:
				print "We couldn't start the daemon. Try to start it manually and then run the GUI again"
				sys.exit(-1)
			else:
				# If we have created more than one daemon, we kill them
				for i in range(1,len(pid)-1):
					os.kill(pid[i],SIG_KILL)
	
	
	def __handler(self):
		self.__backendRunning = True

	def __signalAwaiter(self):
		while (not self.__backendRunning):
			pass
		print "BACKEND INICIALIZADO"
	
	## @brief Gets the list of "must be watched" files.
	# 
	# @see __mergeFileList
	# @return The list of files that must be watched/backup-ed
	# @author Danieloop
	def getWatchedFiles(self):
		return self.__server.getWatchedFiles()
	

	## @brief Overwrite the contents of the lists and the config file and forces new merge.
	#
	# Set the contents of the lists to the user defined ones, and updates the config file with them.
	# Then it forces a new merge of the lists and a search of the patterns.
	# @param whitelist New set of files the user want to watch
	# @param reWhitelist New set of patterns the user want to watch
	# @param blacklist New set of files the user doesnt want to watch
	# @param reBlacklist New set of patterns the user doesnt want to watch.
	# @author Danieloop
	def setFileLists(self, whitelist, reWhitelist, blacklist, reBlacklist):
		return self.__server.setFileLists(whitelist, reWhitelist, blacklist, reBlacklist)

	
	##
	# @author Danieloop
	def getPReWhiteList(self):
		return self.__server.getPReWhiteList()

	##
	# @author Danieloop
	def getPReBlackList(self):
		return self.__server.getPReBlackList()

	##
	# @author Danieloop
	def getPWhiteList(self):
		return self.__server.getPWhiteList()

	##
	# @author Danieloop
	def getPBlackList(self):
		return self.__server.getPBlackList()

	## @brief Gets the Parents List
	#
	# Returns the list of the paths that have been expanded
	# and removed from the whitelist, so Inotify can add them
	# no recursively.
	# @see parentsList
	# @return A Set containing the expanded paths
	# @author Danieloop
	def getParentsList(self):
		return self.__server.getParentsList()

	## @brief Obtains a list of the options in the section especified
	#
	# Returns in a list the options defined in the section especified.
	# @param section The section to retrieve the options from
	# @return A list filled with the options names contained in the section.
	# @author Danieloop
	def getOptions(self,section):
		return self.__server.getOptions(section)

	## @brief Obtains a list of the sections in the config file.
	#
	# Returns in a list the sections available in the config file.
	# @return A list filled with the sections names contained in the file.
	# @author Danieloop
	def getSections(self):
		return self.__server.getSections()

	## @brief Determines if the section given has te option especified
	#
	# Search in the given section for the option passed as argument and
	# return a boolean.
	# @param section The section that may contain de option.
	# @param option The searched option.
	# @return True if the option exists whithin the given section and 
	# False Otherwise
	# @author Danieloop
	def hasOption(self,section,option):
		return self.__server.hasOption(section,option)

	## @brief Determines if the section given exists in the config file.
	#
	# Search in the config file for the section passed as argument and
	# return a boolean.
	# @param section The section the caller wants to know if exists.
	# @return True if the section exists and False Otherwise
	# @author Danieloop
	def hasSection(self,section):
		return self.__server.hasSection(section)

	## @brief Delete a config param from the config file.
	#
	# Remove the option especified from the section given in
	# the config file (dynamic and static).
	# @param section The section containing the option
	# @param option The option to be deleted
	# @throw NoSectionError The section especified doesn't exist.
	# @author Danieloop
	def removeConfigItem(self,section,option):
		return self.__server.removeConfigItem(section,option)

	## @brief Delete a section from de config file
	#
	# Remove a section of the config file if it exists.
	# @param The section to remove
	# @return True if the section really exists and went deleted properly.
	# False otherwise.
	# @author Danieloop
	def removeSection(self,section):
		return self.__server.removeSection(section)

	## @brief Add new section to the config file
	#
	# This method add a new section to the dynamic structures and
	# writes it to the config file..
	# @param section The name of the section that will be added
	# @throw DuplicateSectionError The section especified already exists!
	# @author Danieloop
	def addSection(self,section):
		return self.__server.addSection(section)



	def getLanguage(self):
		return self.__server.getLanguage()


	def setLanguage(self,lang):
		self.__server.setLanguage(lang)


	def getNautilusIntegration(self):
		return self.__server.getNautilusIntegration()


	def setNautilusIntegration(self,nautint):
		self.__server.setNautilusIntegration(nautint)


	def getPeriodically(self):
		return self.__server.getPeriodically()


	def setPeriodically(self,period):
		self.__server.setPeriodically(period)


	def getEveryChange(self):
		return self.__server.getEveryChange()


	def setEveryChange(self,every):
		self.__server.setEveryChange(every)


	def getDeleteIfSpace(self):
		return self.__server.getDeleteIfSpace()


	def setDeleteIfSpace(self,delifspace):
		self.__server.setDeleteIfSpace(delifspace)


	def getDeleteIfOlder(self):
		return self.__server.getDeleteIfOlder()


	def setDeleteIfOlder(self,delifolder):
		self.__server.setDeleteIfOlder(delifolder)


	def getBackupDirectory(self):
		return self.__server.getBackupDirectory()

  
	def setBackupDirectory(self,backupdir):
		self.__server.setBackupDirectory(backupdir)


	def getStartDir(self):
		return self.__server.getStartDir()


	def setStartDir(self,startdir):
		self.__server.setStartDir(startdir)


	def getBlackList(self):
		return self.__server.getBlackList()


	def setBlackList(self,blacklist):
		self.__server.setBlackList(blacklist)


	def getWhiteList(self):
		return self.__server.getWhiteList()


	def setWhiteList(self,whitelist):
		self.__server.setWhiteList(whitelist)


	def getReWhiteList(self):
		return self.__server.getReWhiteList()


	def setReWhiteList(self,rewhitelist):
		self.__server.setReWhiteList(rewhitelist)


	def getReBlackList(self):
		return self.__server.getReBlackList()


	def setReBlackList(self,reblacklist):
		self.__server.setReBlackList(reblacklist)


	def getBatteryChargeLimit(self):
		return self.__server.getBatteryChargeLimit()


	def setBatteryChargeLimit(self,battchargelimit):
		self.__server.setBatteryChargeLimit(battchargelimit)


	def getCpuUseLimit(self):
		return self.__server.getCpuUseLimit()


	def setCpuUseLimit(self,cpuuselim):
		self.__server.setCpuUseLimit(cpuuselim)


	def getMaxBackupSize(self):
		return self.__server.getMaxBackupSize()


	def setMaxBackupSize(self,maxbackupsize):
		self.__server.setMaxBackupSize(maxbackupsize)


	## 
	# @author Danieloop
	#def cronCommand(self):
	#	return self.__serverWatcher.cronCommand()


	## @brief this function is used to share db information with the frontend
	#
	# @author Danieloop
	def getInfo(self):
		return self.__serverScheduler.getInfo()
	

	def recoverSnapshot(self, snapTimestamp, recoverPath):
		return self.__serverSnapshotManager.recoverSnapshot(snapTimestamp, recoverPath)

	
	def recoverBackups(self, snapTimestamp, originalPathList, recoverPath):
		return self.__serverSnapshotManager.recoverBackups(snapTimestamp, originalPathList, recoverPath)
	
	def getCompleteDirList(self, path):
		return self.__serverSnapshotManager.getCompleteDirList(path)

	def getDirList(self, snapTimestamp, path):
		return self.__serverSnapshotManager.getDirList(snapTimestamp,path)	
	
	def getFileInfo(self, snapTimestamp, path):
		return self.__serverSnapshotManager.getFileInfo(snapTimestamp,path) 
	
	def getOldHistorical(self, path): 
		return self.__serverSnapshotManager.getHistorical(path)
	
	def getHistorical(self, path): 
		return self.__serverSnapshotManager.getVersionsBriefing(path)
	
	def lastTimestamp(self):
		return self.__serverSnapshotManager.lastTimestamp()
	
	
	def firstTimestamp(self):
		return self.__serverSnapshotManager.firstTimestamp()
	
	
	def prevTimestamp(self, snapTimestamp):
		return self.__serverSnapshotManager.prevTimestamp(snapTimestamp)
	
	
	def nextTimestamp(self, snapTimestamp):
		return self.__serverSnapshotManager.nextTimestamp(snapTimestamp)
	
	
	def getTimestampsBetween(self,startSnapTimestamp,endSnapTimestamp):
		return self.__serverSnapshotManager.getTimestampsBetween(startSnapTimestamp,endSnapTimestamp)
	
	
	##@brief Gets GMT and DST for a given timestamp
	#
	#@param timestamp reference timestamp
	#@return a tuple with GMT and DST for a nearest timestamp to the given
	# timestamp. If GMT and DST are both -1, timestamp has no attributes.
	def getTimeAttributes(self, timestamp):
		"""
		Gets GMT and DST for a given timestamp
		"""
		return self.__serverSnapshotManager.getTimeAttributes(timestamp)
	
	def stopDaemon(self):
		return self.__server.stopDaemon()
	
	def snapshotRequest(self,user):
		return self.__serverWatcher.snapshotRequest(user)

	def deleteVersions(self,path,init_timestamp,final_timestamp):
		return self.__serverSnapshotManager.deleteBackups(path,init_timestamp,final_timestamp)

	def deleteVersionsBetween(self,path,init_timestamp,final_timestamp):
		return self.__serverSnapshotManager.deleteBackups(path,init_timestamp,final_timestamp)

	def deleteTheseVersions(self,path,timestamps):
		for timestamp in timestamps:
			self.__serverSnapshotManager.deleteBackups(path,timestamp,timestamp)
	
	def deleteAllVersions(self,path):
		return self.__serverSnapshotManager.deleteBackups(path,0,0)
