#!/usr/bin/env python
# -*- coding: utf-8 -*-

#Copyright (C) 2008 Federico Mon Trotti, Daniel Tabas, Carlos Agra, Roberto Cano

#This file is part of HDLorean.
#
#HDLorean is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 2 of the License.

#HDLorean is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with HDLorean.  If not, see <http://www.gnu.org/licenses/>.


import sys
import os
import Close
import CloseAndStop
import Preferences
import DeleteOne
import DeleteSome
import RestWithoutOver
import RestOverwriting
#TODO mirar con cuidado (creo que sobra)
#import SaveBackup
import Search
import FromHD
import FromCD
import FromPC
import ToHD
import ToCD
import ToPC
import About
import Help
import List
import time
import FrontendProxy
import pygtk
import gtk
import gtk.glade
import gnome.ui
import calendar
import gobject
import dbus, dbus.glib, dbus.service
import DebugWindow
import traceback
import urllib
import pynotify
import gettext

class MainPage:
		
	__progressbar = gtk.ProgressBar(adjustment=None)
	__allFilesList=None
	__ipath=None
	__idir=None
	__itime=None
	__sysTrayIcon=None
	##@brief Flag which shows if the hdloreand.py daemon has been started
	#
	#
	__daemonStarted=False

	## Progress bar in status bar class
	#
	# In this class there are some functions to manage the progress bar
	# @author Roberto Cano
	class ProgressBar:
		
		##@brief Update the value of the progress bar so that we get
		# some movement.
		def progress_timeout(pbobj):
			if pbobj.activity_check.get_active():
				pbobj.pbar.pulse()
			else:
				# Calculate the value of the progress bar using the
				# value range set in the adjustment object
				new_val = pbobj.pbar.get_fraction() + 0.01
				if new_val > 1.0:
					new_val = 0.0
					# Set the new value
				pbobj.pbar.set_fraction(new_val)

			# As this is a timeout function, return TRUE so that it
			# continues to get called
			return True

	
	## Dual paned view containing all files recoverable through HD lorean
	#
	# Using a HPane. 
	# @author Ezequiel Lara Gómez and Daniel Tabas
	class AllFilesList:
		
		# Paths TreeStore and treeview
		__pathTreeStore = None
		__pathTreeView = None
		__selfMenu1 = None
		__wTree = None
		##@brief Rows expanded.
		#
		#
		__rowsExpanded = []
		##@brief Path row selected
		#
		#
		__pathRowSelected = None
		##@brief Rows selected by the user.
		#
		#
		__rowsSelected = []
		# For a file, snapshots information ListStore and TreeView	
		__backupsListStore = None
		__backupsTreeView = None	
		__nPath = None	
		##@brief List of paths passed to Preferences constructor to refresh properly.
		#
		#
		__pathPref = []
		# For __inDepth algorithm
		__dicTree = {}
		##@brief Type of targets for the drag'n drop.
		#
		#
		__targets = [('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0),('text/plain', 0, 1),('TEXT', 0, 2),('STRING', 0, 3),]	
		__backupsForLabel = None
		__selectedFileLabel = None
		__main = None
		##@brief Constructor.
		# 
		# @param self
		# @param wTree The wTree containing the XML tree for the Glade file, passed from the MainPage
		# @param proxy The dbus-proxy used to connect to the HD Lorean backend
		# The constructor initializes the ListStore and TreeView components, and loads from the given wTree 
		# to avoid reopening the glade file		
		def __init__(self,wTree,proxy,main):			
			APP='MainPage'
			DIR='../../share/locale/'
			gettext.bindtextdomain(APP, DIR)
			gettext.textdomain(APP)
			_ = gettext.gettext
			gtk.glade.bindtextdomain(APP, DIR)
			gtk.glade.textdomain(APP)
			self.__main = main
			self.__wTree = wTree
			self.refreshTree(wTree,proxy,True)

		##@brief gets the backupliststore.
		#
		# @param self
		# @return returns the backupsliststore object
		# @author Daniel Tabas
		def getBackupsListStore(self):
			return self.__backupsListStore
		
		##@brief gets the backupstreeview.
		#
		# @param self
		# @return returns the backupstreeview object
		# @author Daniel Tabas		
		def getBackupsTreeView(self):
			return self.__backupsTreeView

		##@brief Populate __dicTree with backward recursivity
		#
		# @param self
		# @param path path which is being processed.
		def __processPath(self,path):
			parent, name = os.path.split(path)
			if name:
				self.__processPath(parent)
				if path not in self.__dicTree:
					if os.path.isfile(path):
						self.__dicTree[path] = self.__pathTreeStore.append(self.__dicTree[parent],[self.__get_icon_pixbuf('STOCK_FILE'),name,path])
					elif os.path.isdir(path):
						self.__dicTree[path] = self.__pathTreeStore.append(self.__dicTree[parent],[self.__get_icon_pixbuf('STOCK_DIRECTORY'),name,path])
					else: 
						self.__dicTree[path] = self.__pathTreeStore.append(self.__dicTree[parent],[self.__get_icon_pixbuf('STOCK_SAVE'),name,path])
			else:
				if path not in self.__dicTree:
					if os.path.isfile(path):
						self.__dicTree[path] = self.__pathTreeStore.append(None,[self.__get_icon_pixbuf('STOCK_FILE'),parent,path])
					elif os.path.isdir(path):
						self.__dicTree[path] = self.__pathTreeStore.append(None,[self.__get_icon_pixbuf('STOCK_DIRECTORY'),parent,path])
					else:
						self.__dicTree[path] = self.__pathTreeStore.append(None,[self.__get_icon_pixbuf('STOCK_SAVE'),parent,path])
		
		##@brief Buid the file tree of monitorized system
		#
		# @param self
		# @param list
		def __inDepth(self,list):
			self.__dicTree = {}
			for entry in list:
				self.__processPath(entry[0])
						
		##@brief Refresh the treeview of the left.
		#
		# @param self
		# @param wTree The glade object.
		# @param proxy Dbus proxy to access to backend functions.
		# @param first Flag to show the function if it's the first time call.
		def refreshTree(self,wTree,proxy,first):
			_ = gettext.gettext
			self.__backupsForLabel = wTree.get_widget("backupsForLabel")
			self.__selectedFileLabel = wTree.get_widget("selectedFileLabel")
			#Paths
			self.__pathTreeStore = gtk.TreeStore(gtk.gdk.Pixbuf, str,str)
			self.__pathTreeView = wTree.get_widget("pathTreeView")
			self.__pathTreeView.set_model(self.__pathTreeStore)
		
			self.__backupsListStore = gtk.ListStore(str,str,str,str,str,str,str,int,str)
			self.__backupsTreeView = wTree.get_widget("snapshotsTreeView")
			self.__backupsTreeView.set_reorderable(False)
			self.__backupsTreeView.set_model(self.__backupsListStore)
			self.__backupsTreeView.set_rubber_banding(True)
			self.__backupsTreeView.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
			
			#Enable the selection callbacks
			__pathSel = self.__pathTreeView.get_selection()
			__pathSel.connect('changed', self.on_pathTree_selection_changed)
			
			__snapshotSel = self.__backupsTreeView.get_selection()
			__snapshotSel.connect('changed', self.on_snapshotList_selection_changed)
		
			#TODO refactor to use TreeViewColumn and append_column
			# create the TreeViewColumn to display the data
			if first:
				self.__pathTreeView.enable_model_drag_source( gtk.gdk.BUTTON1_MASK,self.__targets,gtk.gdk.ACTION_DEFAULT|gtk.gdk.ACTION_MOVE)
        			self.__pathTreeView.enable_model_drag_dest(self.__targets,gtk.gdk.ACTION_DEFAULT)
        			self.__pathTreeView.connect("drag_data_received",self.dragDataReceivedData)
				self.__pathTreeView.connect("row-expanded",self.rowExpanded)
				self.__pathTreeView.connect("row-collapsed",self.rowCollapsed)
				self.__pathTreeView.connect("button-press-event",self.pathPressCb)
				self.__backupsTreeView.connect("button-press-event",self.buttonPressCb)
				self.__pathCol = gtk.TreeViewColumn(_('Path'))
				render_pixbuf = gtk.CellRendererPixbuf()
        			self.__pathCol.pack_start(render_pixbuf, False)
        			self.__pathCol.add_attribute(render_pixbuf, 'pixbuf', 0)
        			render_text = gtk.CellRendererText()
        			self.__pathCol.pack_start(render_text, expand=False)
        			self.__pathCol.add_attribute(render_text, 'text', 1)        			
		
				# add tvcolumn to treeview
				self.__pathTreeView.append_column(self.__pathCol)
		
				# create a CellRendererText to render the data
				self.__pathCell = gtk.CellRendererText()
		
				# add the cell to the tvcolumn and allow it to expand
				self.__pathCol.pack_start(self.__pathCell, True)
		
				# set the cell "text" attribute to column 0 - retrieve text
				# from that column in __pathTreeStore
				# FIXME by Karl: eliminating Bug #212765 by deleting this instruction. La dejo comentada un tiempo por si acaso
				#self.__pathCol.add_attribute(self.__pathCell, 'text', 0)
				self.__snapshotsCol = gtk.TreeViewColumn(_('ID'),gtk.CellRendererText(), text=0)
				self.__backupsTreeView.append_column(self.__snapshotsCol)		
		
				
				self.__snapshotsCol = gtk.TreeViewColumn(_('Date'),gtk.CellRendererText(), text=1)
				self.__backupsTreeView.append_column(self.__snapshotsCol)		
		
				self.__snapshotsCol = gtk.TreeViewColumn(_('Size'),gtk.CellRendererText(), text=2)
				self.__backupsTreeView.append_column(self.__snapshotsCol)		
		
				self.__snapshotsCol = gtk.TreeViewColumn(_('Backup size'),gtk.CellRendererText(), text=3)
				self.__backupsTreeView.append_column(self.__snapshotsCol)	
				
				self.__snapshotsCol = gtk.TreeViewColumn(_('Compression Ratio'),gtk.CellRendererText(), text=4)
				self.__backupsTreeView.append_column(self.__snapshotsCol)
				
				self.__snapshotsCol = gtk.TreeViewColumn(_('Operation'),gtk.CellRendererText(), text=5)
				self.__backupsTreeView.append_column(self.__snapshotsCol)	

				self.__snapshotsCol = gtk.TreeViewColumn(_('Modification in'),gtk.CellRendererText(), text=6)
				self.__backupsTreeView.append_column(self.__snapshotsCol)	

			self.__proxy = proxy
			rootDir = self.__proxy.getCompleteDirList("/")
			self.__inDepth(rootDir)
			self.__expandPaths()
			self.__selectOldPath()

		## @brief Callback to the signal press button in the right treeview, in order to popup a menu.
		#
		#
		# @param self
		# @param widget
		# @param event
		# @return returns a boolean, to stop propagating the signal(para que no deseleccione vamos).
		# @author Daniel Tabas
		def buttonPressCb(self,widget,event):
			_ = gettext.gettext
			if event.button == 3:
				self.__menu1 = gtk.Menu()
				menuItem = gtk.ImageMenuItem(_('Restore'))
 				img = gtk.image_new_from_stock(gtk.STOCK_UNDO, gtk.ICON_SIZE_MENU)
 				menuItem.set_image(img)
				menuItem.connect('activate', self.restCb,"Restore")
				self.__menu1.append(menuItem)
				menuItem = gtk.ImageMenuItem(_('Delete'))
	 			img = gtk.image_new_from_stock(gtk.STOCK_DELETE, gtk.ICON_SIZE_MENU)
	 			menuItem.set_image(img)
				menuItem.connect('activate', self.delCb,"Delete")
				self.__menu1.append(menuItem)
				self.__menu1.show_all()
				self.__menu1.popup(None, None, None, 3, event.time)
				return True
		
		## @brief Callback to the signal press button in the right treeview, in order to popup a menu.
		#
		#
		# @param self
		# @param widget
		# @param event
		# @return returns a boolean, to stop propagating the signal(para que no deseleccione vamos).
		# @author Daniel Tabas
		def pathPressCb(self,widget,event):
			_ = gettext.gettext
			if event.button == 3:
				self.__menu1 = gtk.Menu()
				menuItem = gtk.ImageMenuItem(_('Make Backup'))
 				img = gtk.image_new_from_stock(gtk.STOCK_SAVE, gtk.ICON_SIZE_MENU)
 				menuItem.set_image(img)
				menuItem.connect('activate', self.backupCb,"Make Backup")
				self.__menu1.append(menuItem)
				self.__menu1.show_all()
				self.__menu1.popup(None, None, None, 3, event.time)
				return True

		## @brief Callback to save menu item
		#
		# @param self
		# @param widget
		# @param data
		# @author Daniel Tabas
		def backupCb(self,widget, data = None):
			if data:
				self.__main.saveBackup(self.__wTree)

		## @brief Callback to restore menu item
		#
		# @param self
		# @param widget
		# @param data
		# @author Daniel Tabas
		def restCb(self,widget, data = None):
			if data:
				self.__main.restWithoutOver(self.__wTree)

		## @brief Callback to delete menu item
		#
		# @param self
		# @param widget
		# @param data
		# @author Daniel Tabas
		def delCb(self,widget, data = None):
			if data:
				self.__main.deleteOne(self.__wTree)


		## @brief Method that re-expand the rows that where expanded before the treeview collapsed
		#
		# @param self
		# @author Carlos Agra
		def __expandPaths(self):
			for i in self.__rowsExpanded:
				self.__pathTreeView.expand_to_path(i)
				
		## @brief Method that re-select the row that was selected before the treeview collapsed
		#
		# @param self
		# @author Carlos Agra
		def __selectOldPath(self):
			if self.__pathRowSelected != None:
				self.__pathTreeView.get_selection().select_path(self.__pathRowSelected)						
		
		#FIXME by Karl: método copy paste de otra aplicacion, para hacer trastadas con iconos
		## Select an icon from the stock items of gtk		
		# 
		# @param self
		# @param stock The concrete icon from the stock that we will return
		def __get_icon_pixbuf(self, stock):
			return self.__pathTreeView.render_icon(stock_id=getattr(gtk, stock),
                                size=gtk.ICON_SIZE_MENU,
                                detail=None)	
		
		##@brief update the listModel containing the snapshots info.
		# 	
		# For a given path, it checks through the proxy and updates the shown info, activating
		# the label if we have information to show as to provide some feedback. 
		#
		# @param self 
		# @param selected The selected file path, given in a string. 
		# @author Carlos Agra
		def updateSnapshotModel(self,selected):
			# Extracts and compacts into util fields, all info given by
			# getHistorical()
			historical, versionCount = self.__proxy.getHistorical(selected)
			
			self.__backupsListStore.clear()
			if historical: 
				self.__backupsForLabel.set_sensitive(True)
				self.__selectedFileLabel.set_sensitive(True)
				identifier = len(historical)
				for info in historical:
					# Select size of all backed up data for size (or, if it cant be) uses
					# just the size of most recent backed up data (info[4])
					actualSize = info[4]
					if info[5] != -1:
						actualSize = info[5]
					if not actualSize:
						ratio = 'inf'
					else:
						aux = (actualSize - info[3])\
									 / float(actualSize) * 100
						ratio = ("%2.2f" % aux) + "%"
						
					operation = info[2]
					operationPath = info[1]
					if operationPath != selected:
						operation = 'VARIOUS'
						operationPath = '...'
						
					self.__backupsListStore.append(
						[	str(identifier), 
							self.__getHumanTimestamp(info[0]),
							self.convertHumanReadable(actualSize),
							self.convertHumanReadable(info[3]),
							ratio,
							operation,
							operationPath,
							info[0],
							selected
						])
					identifier = identifier - 1

					
		##@brief Given a timestamp in history, the method returns a human readable string for it
		#
		#@param snapTimestamp timestamp to convert into
		#@return a human readable string for timestamp given
		def __getHumanTimestamp(self, snapTimestamp):
			# Get time
			notGMT, isDST = self.__proxy.getTimeAttributes(snapTimestamp)
			humanTimeStamp = time.gmtime(snapTimestamp - notGMT)
			humanTimeStamp = list(humanTimeStamp[0:6])
			timeFormat = '%02d-%02d-%02d %02d:%02d:%02d'
			# Calculates GMT difference and add it to timeString		
			if notGMT: 
				GMT = -(notGMT/3600)
				#timeFormat += ' GMT%+d' % GMT
			# If it is in DST, add it to timestring
			if isDST:
				#timeFormat += ' (DST)'
				humanTimeStamp[3] += 1
			timeString = timeFormat % tuple(humanTimeStamp)
			return timeString
		
		##@brief Converts into util fields all info returned by getHistorical()
		# function.
		#
		# @param allInfo all info given by self.__proxy.getHistorical(path)[0]
		# @param path of versions
		# @return a compacted and util historical
		# @author Salvador de la Puente
		def __filterInfo(self, allInfo, path):
			"""
			Converts into util fileds all info returned by getHistorical()
			function.
			"""
			historical = {}
			oldTimestamp = None
			for info in allInfo:
				actualTimestamp = info[1]
				
				if oldTimestamp != actualTimestamp:
					historical[actualTimestamp] = \
					{'timestamp':actualTimestamp, 'size':0, 'bsize':0,
					 'operation':None, 'operation path':None}
				
				# Discard directories
				if info[16] != "D":
					historical[actualTimestamp]['size'] = \
									historical[actualTimestamp]['size'] + info[9]
					historical[actualTimestamp]['bsize'] = \
									historical[actualTimestamp]['bsize'] + info[7]
				
				# Operation
				if not historical[actualTimestamp]['operation']:
					historical[actualTimestamp]['operation'] = info[5]
				else:
					historical[actualTimestamp]['operation'] = 'VARIOUS'
					
				# Operation path
				if not historical[actualTimestamp]['operation path']:
					historical[actualTimestamp]['operation path'] = info[3]
				else:
					historical[actualTimestamp]['operation path'] = '...'
								
				oldTimestamp = actualTimestamp
						
			return historical
		
		## @brief Convert a given bytes size to human readable size.
		# 	
		# @param self
		# @param size The size to convert.
		# @author Carlos Agra
		def convertHumanReadable(self,size):
			size = long(size)
			if size < 1024.0:
				size = str(size)					
				size = size + ' B'
				return size			
			size = size / 1024.0
			if size > 1024.0:
				size = size / 1024.0
				i = 0
				tam = ''
				size = str(size)
				while size[i] != '.':
					tam = tam + size[i]
					i = i + 1
				tam = tam + size [i] + size [i+1]					
				size = tam + ' MB'
			else:
				i = 0
				tam = ''
				size = str(size)
				while size[i] != '.':
					tam = tam + size[i]
					i = i + 1
				tam = tam + size [i] + size [i+1] 
				size = tam + ' KB'
			return size
	
		## @brief Callback for a selection changed event in the paths tree.
		#
		# When a selection in the treeView is changed, we get the file path and update the snapshots shown for it.
		#	
		# @param self
		# @param selection The new GTK selection object
		def on_pathTree_selection_changed(self, selection):
			model, selection_iter = selection.get_selected()
			self.__backupsForLabel.set_sensitive(False)		
			self.__selectedFileLabel.set_sensitive(False)
		
			if (selection_iter and model):
				#Something is selected so get the object
				selected = model.get_value(selection_iter, 2)
				#updated selected info on the right pane
				self.__selectedFileLabel.set_text(selected)
				selected = model.get_value(selection_iter, 2)
				self.__nPath = selected
				self.updateSnapshotModel(selected)
			else: 
				self.__selectedFileLabel.set_text("")

		def getNPath(self):
			return self.__nPath
		
		## brief Callback for a selection changed event in the snapshots tree.
		#
		#This function stores the paths of the rows of the liststore which has been
		#selected by the user. 
		#
		# @param self
		# @param selection The new GTK selection object.
		# @author Daniel Tabas
		def on_snapshotList_selection_changed(self,selection):
			self.__rowsSelected = []
			self.__rowsSelected = selection.get_selected_rows()[1]		
		

		## @brief getter for the __rowsSelected atribute
		#
		# @param self
		# @return It returns a list of paths of the selected rows.
		# @author Daniel Tabas	
		def getRowsSelected(self):
			return self.__rowsSelected
		
		## @brief Getter of a field of one of the selected rows.
		#
		# @param self
		# @param row Integer to access to the __rowSelected list.
		# @param element Field of the row.
		# @return It returns the field selected of the row.
		# @author Daniel Tabas
		def getColumnSelected(self,row,element):
			return self.__backupsTreeView.get_model().get_value(self.__backupsTreeView.get_model().get_iter(self.__rowsSelected[row]),element)

		## @brief Method called when the drag_data_received event is captured, in order to do some drag & drop.
		#
		# @param self
		# @param treeview treeview in wich we capture drag_data_received event
		# @param context context of the treeview
		# @param x horizontal position.
		# @param y vertical position.
		# @param selection info about the dragged object
		# @param info more info about the object
		# @param etime timestamp info of the drag
		# @author Daniel Tabas
		def dragDataReceivedData(self, treeview, context, x, y, selection,info, etime):
			uri = selection.data.strip()
			uri_splitted = uri.split()
			for uri in uri_splitted:
				if os.path.isdir(uri.replace("file://","")) or os.path.isfile(uri.replace("file://","")):
					path = urllib.unquote_plus(uri.replace("file://",""))
					self.__pathPref.append(path.encode("utf-8","strict").replace('\n',"").replace('\t',""))
					lista = self.__proxy.getWhiteList()
					if lista == "":
						lista = path
					else:
						lista= lista.replace('\n',"").replace('\t',"") +";"+path
					l1=lista.replace('\n',"").replace('\t',"")
					self.__proxy.setWhiteList(l1.encode("utf-8","strict"))
			return
		
		## @brief Method called when a row in the path treeview is expanded, to store the information.
		#
		# @param self
		# @param treeview treeview in wich we capture row_expanded event
		# @param iter the iter of the row that is expanded
		# @param path the path that is expanded
		# @author Carlos Agra
		def rowExpanded(self,treeview, iter, path):
			newPath = str()
			j=len(path)
			i=0
			while i != j:
				number = str(path[i])
				if i != j-1:
					number = number + ":"
				newPath = newPath + number
				i=i+1
			isInList = self.__rowsExpanded.count(newPath)
			if isInList == 0:
				self.__rowsExpanded.append(newPath)
				
		## @brief Method called when a row in the path treeview is Collapsed, to store the information.
		#
		# @param self
		# @param treeview treeview in wich we capture row_expanded event
		# @param iter the iter of the row that is expanded
		# @param path the path that is expanded
		# @author Carlos Agra
		def rowCollapsed(self,treeview, iter, path):
			newPath = str()
			j=len(path)
			i=0
			while i != j:
				number = str(path[i])
				if i != j-1:
					number = number + ":"
				newPath = newPath + number
				i=i+1
			self.__rowsExpanded.remove(newPath)
			itemsToDelete = []
			for item in self.__rowsExpanded:
				if item.startswith(newPath):
					itemsToDelete.append(item)
			for item in itemsToDelete:
				self.__rowsExpanded.remove(item)
			
		

		## @brief Returns the path dragged into the treeview.
		#
		# @param self
		# @return Returns an string with the path dragged
		# @author Daniel Tabas
		def get_path(self):
			return self.__pathPref

		## @brief Empty the list in witch we put the paths dragged into the treeview.
		#
		# @param self
		# @author Daniel Tabas
		def set_empty(self):
			self.__pathPref = []

	## @brief The constructor of the main page.
	#
	# The first is loaded in self->wTree.
	# The dictionaty connects the widget's signals with class functions.
	# TODO by Carlos: single instance for all windows
	# @param self The object pointer
	# @author Roberto Cano Fernandez, Daniel Tabas, Carlos Agra and Ezequiel Lara Gómez
	def __init__(self,n):
		APP='MainPage'
		DIR='../../share/locale/'
		gettext.bindtextdomain(APP, DIR)
		gettext.textdomain(APP)
		_ = gettext.gettext
		gtk.glade.bindtextdomain(APP, DIR)
		gtk.glade.textdomain(APP)
		#instance of frontend proxy
		self.__proxy = FrontendProxy.FrontendProxy()
		self.__progressbar.set_orientation(gtk.PROGRESS_LEFT_TO_RIGHT)
		bus = dbus.SessionBus()
		bus.add_signal_receiver(self.__handlerUnexpected,'signalUnexpectedExit','com.hdlorean.Signals','com.hdlorean.Server','/hdlorean/signals')
		bus.add_signal_receiver(self.__handlerRefresh,'makeSnapshot','com.hdlorean.SignalsSch','com.hdlorean.Server','/hdlorean/signalsSch')
		#instance of the status icon		
		self.__initDaemon()
		# Variable wich contains the .glade file.
		self.gladefile="../../share/glade/HDLoreanMainPage.glade"
		self.__nTimesPref = n
		# Contains the XML info of the .glade 
		self.wTree = gtk.glade.XML (self.gladefile)

		self.__allFilesList = self.AllFilesList(self.wTree,self.__proxy,self)

		#disable unimplemented features (TODO!)
		self.wTree.get_widget('checkmenuViewAllSystem').set_sensitive(False)
		self.wTree.get_widget('menuImp3').set_sensitive(False)
		self.wTree.get_widget('menuExp3').set_sensitive(False)
		self.wTree.get_widget('imagemenuSynchronize').set_sensitive(False)
		self.wTree.get_widget('imagemenuSearch').set_sensitive(False)
		self.wTree.get_widget('menuDelVer').set_sensitive(False)
		# get the status bar to add a progress bar.
		self.wTree.get_widget('mainPageStatus')
		
		# dictionary: connects the signals with the Main Window functions
		dic = {"on_imagemenuClose_activate" : self.close,
			"on_menuPreferences_activate" : self.preferences,
			"on_toolPreferences_clicked" : self.preferences,
			"on_imagemenuQuit_activate" : self.quit,
			"on_imagemenuDelOne_activate" : self.deleteOne,
			"on_toolDelete_clicked" : self.deleteOne,
			"on_imagemenuDelSome_activate" : self.deleteSome,
			"on_imagenmenuRestWithOver_activate" : self.restWithoutOver,
			"on_toolRestore_clicked" : self.restWithoutOver,
			"on_imagenmenuRestOver_activate" : self.restOverwriting,
			"on_imagemenuSaveBackup_activate" : self.saveBackup,
			"on_toolSave_clicked" : self.saveBackup,
			"on_imagemenuSearch_activate" : self.search,
			"on_toolSearch_clicked" : self.search,
			"on_imagemenuFromHD_activate" : self.fromHD,
			"on_imagemenuFromCD_activate" : self.fromCD,
			"on_imagemenuFromPC_activate" : self.fromPC,
			"on_imagemenuToHD_activate" : self.toHD,
			"on_imagemenuToCD_activate" : self.toCD,
			"on_imagemenuToPC_activate" : self.toPC,
			"on_imagemenuSynchronize_activate" : self.sync,
			"on_imagemenuAbout_activate" : self.about,
			"on_imagemenuHelp_activate" : self.help,
			"on_toolHelp_clicked" : self.help,
			"on_mainPage_destroy" : self.destroyWind
		}
		#connect class signals using the given dictionary
		self.wTree.signal_autoconnect (dic)
		self.window=self.wTree.get_widget('mainPage')

	## @brief Shows information related to HDLorean in a popup
	# 
	# @param self
	# @param messages the list of messages to show, where messages[0] will be the title and the remaining elements the paths modified
	# @author Carlos Agra

	def showMessage(self,messages):
		if pynotify.init("HD Lorean notify"):
			numMessages=len(messages)
			if numMessages >= 1:
				paths = str()
				i = 1
				while i != numMessages:
					paths = paths + messages[i]
					if i != numMessages -1:
						paths = paths + "\n"
					i=i+1								
	        		n = pynotify.Notification(messages[0], paths,gtk.STOCK_INFO)
				n.attach_to_status_icon(self.__sysTrayIcon)
        			n.show()
    		else:
        		print _("there was a problem initializing the pynotify module")


	## @brief Handler for the unexpected exit signal.
	# 
	# @param self
	# @param trace string with the trace to show in the debug window.
	# @author Daniel Tabas
	def __handlerUnexpected(self,trace):	
		strtrace="python ./InitDebug.py " +'"' +str(trace)+'"'
		tr = strtrace.replace('\n'," linea")
		os.system(tr)	

	## @brief Handler for the refresh signal.
	# 
	# @param self
	# @author Daniel Tabas
	def __handlerRefresh(self):
		self.__allFilesList.refreshTree(self.wTree,self.__proxy,False)

	## @brief Function that creates the systray icon.
	#
	# @param self
	# @author Daniel Tabas
	def __initDaemon(self):
		_ = gettext.gettext
		if self.__daemonStarted==False:
			self.__sysTrayIcon = gtk.StatusIcon()
			#gtk menu of the systray icon
			self.__menu = gtk.Menu()
			menuItem = gtk.ImageMenuItem(_('Close'))
 			img = gtk.image_new_from_stock(gtk.STOCK_QUIT, gtk.ICON_SIZE_MENU)
 			menuItem.set_image(img)
			menuItem.connect('activate', self.quitCb,  self.__sysTrayIcon)
			self.__menu.append(menuItem)
			menuItem = gtk.ImageMenuItem(_('Stop Daemon and Exit'))
 			img = gtk.image_new_from_stock(gtk.STOCK_STOP, gtk.ICON_SIZE_MENU)
 			menuItem.set_image(img)
			menuItem.connect('activate', self.stopCb, self.__sysTrayIcon)
			self.__menu.append(menuItem)
			self.__sysTrayIcon.connect('activate', self.activateIconCb, self.__sysTrayIcon)
			self.__sysTrayIcon.connect('popup-menu', self.popupMenuCb, self.__menu)
			self.__sysTrayIcon.set_from_file("../../share/images/HDLoreanHumo.png")
			self.__sysTrayIcon.set_visible(True)
			self.__sysTrayIcon.set_tooltip(_("HD Lorean Icon"))	
			self.__daemonStarted=True

	## @brief handler for the quit menu.
	#
	# @param self
	# @param widget
	# @param data
	# @author Daniel Tabas
	def quitCb(self,widget, data = None):
		if data:
			self.close(widget)

	## @brief handler for the stop menu.
	#
	# @param self
	# @param widget
	# @param data
	# @author Daniel Tabas	
	def stopCb(self,widget,data = None):
		if data:
			self.quit(widget)


	## @brief handler for the left button press in the systray icon.
	#
	# @param self
	# @param widget
	# @param button
	# @param time
	# @param data
	# @author Daniel Tabas
	def popupMenuCb(self,widget, button, time, data = None):
		if button == 3:
			if data:
				data.show_all()
				data.popup(None, None, None, 3, time)
			pass


	## @brief handler for the stop menu.
	#
	# @param self
	# @param widget
	# @param data
	# @author Daniel Tabas	
	def activateIconCb(self,widget, data = None):
		if data:
			if not self.window.get_property("visible"):		
				self.window.destroy()
				self.__init__(1)
				self.setNTimesPref(1)
			else:
				self.window.destroy()
				self.setNTimesPref(1)

	

	## @brief Put a percentage value in the status bar.
	#
	# This method takes an operation name and a progress percentage and place
	# both in the status bar. The idea is make a callback to this function once
	# per second...
	# @return nothing
	# @author Roberto Cano.
	def putPercentage(self,oper, perc):
		_ = gettext.gettext
		if ((oper>=0) or (oper <=100)):
			self.wTree.get_widget('mainPageStatus').pop(0)
			self.wTree.get_widget('mainPageStatus').push(0,_("Porcentage of ")+oper+_(" operation is: ")+str(perc)+" %.")
	
	
	## @brief Close the app.
	#
	# 
	# @param
	# @return
	# @author
	def close(self,widget): 
		app=Close.Close()


	## @brief Setter for the ntimespref atribute
	#
	# @param self
	# @param nTimes integer to set.
	# @author Daniel Tabas
	def setNTimesPref(self,nTimes):
		self._nTimesPref=nTimes


	## @brief Init for the preferences window.
	#
	# 
	# @param self 
	# @param __nTimesPref Flag to refresh correctly. 
	# @author Carlos Agra
	def preferences(self,widget):
		app=Preferences.Preferences(self.__nTimesPref,self.__allFilesList.get_path())
		self.__allFilesList.set_empty()
		self.__nTimesPref=self.__nTimesPref+1
		
	
	## @brief Function that deletes one o more versions of a file.
	#
	# 
	# @param self
	# @param widget
	# @author Carlos Agra, Daniel Tabas
	def deleteOne(self,widget):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("Delete One Version"))
		index = self.__allFilesList.getRowsSelected()
		tempPath = []
		if len(index)!=0:
			for i in range(0,len(index)):
				init_timestamp = self.__allFilesList.getColumnSelected(i,7)
				path = self.__allFilesList.getColumnSelected(i,8)
				tempPath.append(path)
				finalpath = self.__proxy.deleteVersions(path,init_timestamp,0)
			self.__allFilesList.updateSnapshotModel(path)
			self.__allFilesList.refreshTree(self.wTree,self.__proxy,False)
			self.wTree.get_widget('mainPageStatus').push(0,_("Delete OK!"))
			self.showMessage(['HD Lorean',_('Delete OK of:')]+tempPath)
		else:
			self.wTree.get_widget('mainPageStatus').push(0,_("Please select one row to be deleted"))

	## @brief Stop the daemon and exit.
	#
	# 
	# @param self
	# @param widget
	# @author 
	def quit(self,widget):
		#self.__proxy.stopDaemon()		
		app=CloseAndStop.CloseAndStop(self.__proxy)		

	## @brief
	#
	# This method
	# @param
	# @return
	# @author
	def deleteSome(self,widget):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("Delete Some Versions"))
		app=DeleteSome.DeleteSome()


	## @brief Restore the selected backup (by its timestamp) and put it in the
	# selected folder, overwriting the previous files.
	#
	# @param timestamp The timestamp of the snapshot we want to recover
	# @param r_path The path where the snapshot will be recover
	# @return
	# @author Federico Mon, Roberto Cano
	#FIXME,idir,ipath,itime):
	def restWithoutOver(self,widget):
		_ = gettext.gettext
		#aqui ira la llamada a lo del porcentaje
		# ni guarra de si esto funcionaria asi
		#FIXME if  (ipath is  None) or (idir is None) or (itime is None):
		self.wTree.get_widget('mainPageStatus').pop(0)		
		index = self.__allFilesList.getRowsSelected()
		if len(index)!=0:
			dialog = gtk.FileChooserDialog(_("Recover path"), None, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
			dialog.set_current_folder(os.environ['HOME'])
			if dialog.run() == gtk.RESPONSE_OK:
					directory = dialog.get_filename()
					dialog.destroy()
			else:
					self.wTree.get_widget('mainPageStatus').push(0,_("Restore cancelled"))
					#directory = None
					dialog.destroy()
					return None
			for i in range(0,len(index)): 
				self.wTree.get_widget('mainPageStatus').push(0,_("Restoring..."))
				timestamp = self.__allFilesList.getColumnSelected(i,7)
				path = self.__allFilesList.getColumnSelected(i,8)
				# if there's no input path or input directory to restore open the choose dialog (menu button was pressed)
				finalpath = self.__proxy.recoverBackups(timestamp,[path],directory)
				os.system("xdg-open '"+finalpath+"'")
			self.wTree.get_widget('mainPageStatus').push(0,_("Restore OK!"))
			self.showMessage(['HD Lorean',_('Restore succesful of:'),path])
			#return directory
			
		else:
			self.wTree.get_widget('mainPageStatus').push(0,_("Please select a row of the list"))
	#FIXME else:
		#FIXME finalpath=self.__proxy.recoverBackups(itime,[ipath],idir)
			
	
		#return directory



			#TODO by Carlos: recoger timestamp, r_path. La llamada a RestWithoutOver falla
		#	app=RestWithoutOver.RestWithoutOver()


	## @brief Restore the selected backup (by its timestamp) and put it in the
	# selected folder, overwriting the previous files.
	#
	# @param timestamp The timestamp of the snapshot we want to recover
	# @param r_path The path where the snapshot will be recover
	# @return
	# @author Adrián
	def restOverwriting(self,timestamp, r_path):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("Restore overwritting"))
		model,itt = self.getBackupsTreeView().get_selection().get_selected()
		if (model is not None) and (itt is not None):
			selection = self.__liststoreList.get_value(itt, 0)
			self.__proxy.recoverSnapshot(timestamp,r_path)
			#TODO by Carlos: recoger timestamp, r_path. La llamada a RestWithoutOver falla
			app=RestOverwriting.RestOverwriting()
		#TODO: Move it to overwrite.
	

	## @brief Backup the selected folder or all the watched files.
	# 
	#
	# @param self
	# @param widget
	# @author 
	def saveBackup(self,widget):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("Save backup"))
		nPath = self.__allFilesList.getNPath()
		if nPath is not None:
			#TODO cambiar esto por la funcion nueva que le he pedido a adri
			self.__proxy.snapshotRequest("USER")
			#TODO si falla que pasa? ademas sale esto mucho antes de que termine, tema de hilos
			self.wTree.get_widget('mainPageStatus').push(0,_("Backup saved OK!"))
			self.showMessage(['HD Lorean','Backup of following path OK:',nPath])
		else:
			self.__proxy.snapshotRequest("USER")
			#TODO si falla que pasa? ademas sale esto mucho antes de que termine, tema de hilos
			self.wTree.get_widget('mainPageStatus').push(0,_("Backup saved OK!"))
			self.showMessage(['HD Lorean','Backup done OK'])
	

	## @brief
	#
	# This method
	# @param
	# @return
	# @author
	def search(self,widget):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("Search"))
		app=Search.Search()
	
	
	# @brief
	#
	# This method
	# @param
	# @return
	# @author
	def fromHD(self,widget):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("Import from a Hard Disk Drive"))
		app=FromHD.FromHD()
	# @brief
	#
	# This method
	# @param
	# @return
	# @author
	def fromCD(self,widget):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("Import from a CD or DVD"))
		app=FromCD.FromCD()
	# @brief
	#
	# This method
	# @param
	# @return
	# @author
	def fromPC(self,widget):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("Import from another PC"))
		app=FromPC.FromPC()
	# @brief
	#
	# This method
	# @param
	# @return
	# @author
	def toHD(self,widget):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("Export to an Hard Disk Drive"))
		app=ToHD.ToHD()
	# @brief
	#
	# This method
	# @param
	# @return
	# @author
	def toCD(self,widget):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("Export to a CD or a DVD"))
		app=ToCD.ToCD()
	# @brief
	#
	# This method
	# @param
	# @return
	# @author
	def toPC(self,widget):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("Export to another PC"))
		app=ToPC.ToPC()
	# @brief
	#
	# This method
	# @param
	# @return
	# @author
	def sync(self,widget):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("Synchronize"))
		app=Synchronize.Synchronize()
	# @brief
	#
	# This method
	# @param
	# @return
	# @author
	def about(self,widget):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("About HD Lorean"))
		app=About.About()
	
	# @brief
	#
	# This method
	# @param
	# @return
	# @author
	def help(self,widget):
		_ = gettext.gettext
		self.wTree.get_widget('mainPageStatus').pop(0)
		self.wTree.get_widget('mainPageStatus').push(0,_("Help"))
		app=Help.Help()
		
	# @brief
	#
	# This method
	# @param
	# @return
	# @author
	def destroyWind(self,widget):
		self.window.set_property("visible", False)	

# Main page initialization
if (__name__ == "__main__"):
	try:
		app=MainPage(0)
		gtk.main()
	except KeyboardInterrupt,trace:
		print "GUI has received exiting signal. Shuting down..."
		sys.exit(0)
	except Exception, trace:
		#os.spawnl(os.P_NOWAIT,"./InitDebug.py")
		#FIXME esto es muy feo pero de momento no he encontrado otra cosa que funcione
		strtrace="python ./InitDebug.py " +'"' +str(traceback.format_exc())+'"'
		tr = strtrace.replace('\n'," linea")
		os.system(tr)
		sys.exit(0)
