# -*- coding: utf-8 -*-

#Copyright (C) 2008 Federico Mon Trotti, Daniel Tabas, Carlos Agra, Roberto Cano

#This file is part of HDLorean.
#
#HDLorean is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 2 of the License.

#HDLorean is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with HDLorean.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import subprocess
import FrontendProxy
import AddFileIncPattern
import AddFileExcPattern
import pygtk
import gtk
import gtk.glade
import gnome.ui
import calendar
import gettext

## @brief Menu Preferences.
#
# In this window user can configure HD Lorean, all changes will be writen in a config file
# @ author HD Lorean GUI developers
class Preferences:
	## Class variables they hace been used to capture all events from glade widgets
	#maxbackupsize
	__sizeValue="0"
	#nautilusintegration (bool)
	__nautInt="False"
	#everychange (bool)
	__modBackChange="True"
	#periodically
	__modBackEveryVal="-1"
	#backupdirectory
	__chooseDir=""
	#languaje
	__lang="English(Default)"
	#deleteifspace	
	__spaceLimit="0"
	#deleteifolder
	__timeLimit="0"
	#batterychargelimit
	__batteryLimitValue='0'
	#cpuuselimit	
	__cpuLimitValue='0'
	#checks de patrones
	__incHidden="False"
	__incCnf="False"
	__incConf="False"
	__excHidden="False"
	__excBak="False"
	__excTemp="False"
	__excTilde="False"
	#################
	#listas
	__blacklist=""
	__whitelist=""
	__re_blacklist=""
	__re_whitelist=""
	# luego se las pasamos a add...
	#patrones a mano
	__includePattern=""
	__excludePattern=""
	__nTimesSelf=0
	#TODO nos falta por dar a elegir el start dir???
	## Treeview, atributes of the class
	# List of indexed directories & files
	__itemsAdded = []
	# List of excluded directories & files
	__itemsExcluded = []
	# Liststore & treeview for included directories & files
	__liststoreInclude = gtk.ListStore(str)
	__treeviewInclude = gtk.TreeView (__liststoreInclude)
	# Liststore & treeview for excluded directories & files
	__liststoreExclude = gtk.ListStore(str)
	__treeviewExclude = gtk.TreeView (__liststoreExclude)

	## The constructor.
	#
	# @param self 
	# @param main : indicate the window that calls to Prefences constructor
	# @param __nTimes : indicate when is the first call to Preferences	
	def __init__(self,__nTimes,listaPath):
		APP='Preferences'
		DIR='../../share/locale/'
		gettext.bindtextdomain(APP, DIR)
		gettext.textdomain(APP)
		_ = gettext.gettext
		gtk.glade.bindtextdomain(APP, DIR)
		gtk.glade.textdomain(APP)
		self.__nTimesSelf=__nTimes
		self.__proxy = FrontendProxy.FrontendProxy()		
		# @var gladefile  Variable wich contains the .glade file.
		self.gladefile="../../share/glade/HDLoreanPreferencesMenu.glade"
		# @var wTree Contains the XML info of the .glade 
		self.wTree = gtk.glade.XML (self.gladefile)
		self.__treeviewInclude = self.wTree.get_widget('treeviewInclude')
####################################################################################################################################
		# here is the point where the config file is readed
		########################################
		#	BACKUPINTERVAL
		########################################
		if self.__proxy.hasSection("BackupInterval"):		
			if self.__proxy.hasOption("BackupInterval","everychange"):
				self.__modBackChange=str(self.__proxy.getEveryChange())
				if self.__modBackChange=="1" :
					self.wTree.get_widget('checkModChange').set_active(True)
				else:	
					self.wTree.get_widget('checkModChange').set_active(False)
			if self.__proxy.hasOption("BackupInterval","periodically"):
				self.__modBackEveryVal= self.__proxy.getPeriodically()
				if self.__modBackEveryVal=="5 min":
					self.wTree.get_widget('comboEvery').set_active(0)
				if self.__modBackEveryVal=="10 min":
					self.wTree.get_widget('comboEvery').set_active(1)
				if self.__modBackEveryVal=="30 min":
					self.wTree.get_widget('comboEvery').set_active(2)
				if self.__modBackEveryVal=="1 hour":
					self.wTree.get_widget('comboEvery').set_active(3)
				if self.__modBackEveryVal=="1 day":
					self.wTree.get_widget('comboEvery').set_active(4)
		########################################
		#	LIMITS
		########################################
		if self.__proxy.hasSection("Limits"):
			if self.__proxy.hasOption("Limits","maxbackupsize"):
				self.__sizeValue=self.__proxy.getMaxBackupSize()
				self.wTree.get_widget('sizeOptSpin').set_value(float(self.__sizeValue))
			if self.__proxy.hasOption("Limits","batterychargelimit"):
				self.__batteryLimitValue=self.__proxy.getBatteryChargeLimit()
				self.wTree.get_widget('spinBattery').set_value(float(self.__batteryLimitValue))
			if self.__proxy.hasOption("Limits","cpuuselimit"):
				self.__cpuLimitValue=self.__proxy.getCpuUseLimit()
				self.wTree.get_widget('spinCPU').set_value(float(self.__cpuLimitValue))
		########################################
		#	HDUSAGE
		########################################
		if self.__proxy.hasSection("HDUsage"):		
			if self.__proxy.hasOption("HDUsage","backupdirectory"):
				self.__chooseDir=self.__proxy.getBackupDirectory()
				self.wTree.get_widget('chooserDirectory').set_current_folder(self.__chooseDir)
			if self.__proxy.hasOption("HDUsage","deleteifspace"):
				self.__spaceLimit=self.__proxy.getDeleteIfSpace()
				self.wTree.get_widget('spinSpaceLimit').set_value(float(self.__spaceLimit))
			if self.__proxy.hasOption("HDUsage","deleteifolder"):
				self.__timeLimit=self.__proxy.getDeleteIfOlder()
				self.wTree.get_widget('spinTimeExceed').set_value(float(self.__timeLimit))
		########################################
		#	LISTS
		########################################
		if self.__proxy.hasSection("Lists"):
			self.__treeviewInclude.set_model(None)
			if self.__nTimesSelf == 0:
				self.__nTimesSelf = self.__nTimesSelf+1
				if self.__proxy.hasOption("Lists","blacklist"):
					__aux=self.__proxy.getBlackList()
					__aux2=__aux.split(";")
					i=0
					if __aux2[0]!= "":				
						while i<len(__aux2):
							self.__itemsExcluded.append(__aux2[i])		
							itt = self.__liststoreExclude.append([__aux2[i]])				
							i=i+1
				if self.__proxy.hasOption("Lists","whitelist"):
					__aux=self.__proxy.getWhiteList()				
					__aux2=__aux.split(";")
					i=0
					if __aux2[0]!= "":
						while i<(len(__aux2)):
							self.__itemsAdded.append(__aux2[i])		
							itt = self.__liststoreInclude.append([__aux2[i]])				
							i=i+1
			else:
				aux3 = listaPath
				if len(aux3) >0:
					i=0
					if aux3[0]!= "":
						while i<(len(aux3)):
							self.__itemsAdded.append(aux3[i])		
							itt = self.__liststoreInclude.append([aux3[i]])				
							i=i+1
				else:
					pass#print "LISTA VACIA"
			if self.__proxy.hasOption("Lists","re_blacklist"):
				__aux=self.__proxy.getReBlackList()
				__aux2=__aux.split(";")
				i=0	
				while i<len(__aux2):
					if __aux2[i]=="*/.*": 
						self.wTree.get_widget('checkHiddenExcl').set_active(True)
						self.__excHidden="True"
					if __aux2[i]=="*.bak":
						self.wTree.get_widget('checkBakFilesExcl').set_active(True)
						self.__excBak="True"
					if __aux2[i]=="*.tmp":
						self.wTree.get_widget('checkTempFilesExcl').set_active(True)
						self.__excTemp="True"
					if __aux2[i]=="*~":
						self.wTree.get_widget('checkTildeFilesExcl').set_active(True)
						self.__excTilde="True"
					i=i+1
			if self.__proxy.hasOption("Lists","re_whitelist"):
				__aux=self.__proxy.getReWhiteList()
				__aux2=__aux.split(";")
				i=0
				while i<len(__aux2):
					if __aux2[i]=="*/.*":
						self.__incHidden="True"
						self.wTree.get_widget('checkHidden').set_active(True)
					if __aux2[i]=="*.cnf":
						self.__incCnf="True"
						self.wTree.get_widget('checkCnfFiles').set_active(True)
					if __aux2[i]=="*.conf":
						self.__incConf="True"
						self.wTree.get_widget('checkConfFiles').set_active(True)
					i=i+1
		########################################
		#	GENERAL
		########################################
		if self.__proxy.hasSection("General"):		
			if self.__proxy.hasOption("General","language"):
				#self.__lang=self.__proxy.getLanguage()
				#FIXME Aqui si hubiera mas de un lenguaje en la aplicacion habria que hacer un if
				#self.wTree.get_widget('comboLanguage').set_active(0)
				pass
			if self.__proxy.hasOption("General","nautilusintegration"):
				self.__nautInt=str(self.__proxy.getNautilusIntegration())
				if self.__nautInt=="1" :
					self.wTree.get_widget('checkActNaut').set_active(True)
				else:	
					self.wTree.get_widget('checkActNaut').set_active(False)
		#end of reading of config file
####################################################################################################################################
		## Treeviews initializations
		# Prepare the renderers
		textrenderer = gtk.CellRendererText()
		# Include initializations
		
		self.__treeviewInclude.set_model(self.__liststoreInclude)
		self.__treeviewInclude.set_headers_visible(True)
		# Add column 'Path' to the include treeview
		header, col, renderer = _('Path'), 0, textrenderer
		column = gtk.TreeViewColumn(header, renderer, text=col)
		column.set_sort_column_id(col)
		column.set_resizable(True)
		self.__treeviewInclude.append_column(column)		
		# Exclude initializations
		self.__treeviewExclude = self.wTree.get_widget('treeviewExclude')
		self.__treeviewExclude.set_model(self.__liststoreExclude)
		self.__treeviewExclude.set_headers_visible(True)
		# Add column 'Path' to the exclude treeview
		header2, col2, renderer2 = _('Path'), 0, textrenderer
		column2 = gtk.TreeViewColumn(header2, renderer2, text=col2)
		column2.set_sort_column_id(col2)
		column2.set_resizable(True)
		self.__treeviewExclude.append_column(column2)
		# dictionary, connect window's signals with class functions
		dic = {	"on_CancelButton_clicked" : self.cancel,
			"on_AcceptButton_clicked" : self.accept,
			"on_ApplyButton_clicked" : self.applyB,
			"on_Preferences_destroy" : self.DestroyWind,
			"on_sizeOptSpin_value_changed" : self.setSizeValue,
			"on_checkActNaut_toggled" : self.actNautInt,
			"on_checkModChange_toggled" : self.modBackIntChan,
			"on_comboEvery_changed" : self.setBackIntEvery,
			"on_chooserDirectory_current_folder_changed" : self.setChoosDir,
			#"on_comboLanguage_changed" : self.setLang,
			"on_spinSpaceLimit_value_changed" : self.setSpaceLimit,
			"on_spinTimeExceed_value_changed" : self.setTimeLimit,
			"on_spinBattery_value_changed" : self.setBatteryLimit ,
			"on_spinCPU_value_changed" : self.setCpuLimit ,
			"on_addFileIncButton_clicked" :  self.addFileIncButton,
			"on_addFolderIncButton_clicked" :  self.addFolderIncButton,
			"on_removeIncButton_clicked" :  self.removeIncButton,
			"on_addFileIncPattern_clicked": self.addFileInc,
			"on_checkHidden_toggled" : self.includeHidden,
			"on_checkCnfFiles_toggled" : self.includeCnf,
			"on_checkConfFiles_toggled" : self.includeConf,
			"on_addFileExcButton_clicked" :  self.addFileExcButton,
			"on_addFolderExcButton_clicked" :  self.addFolderExcButton,
			"on_removeExcButton_clicked" :  self.removeExcButton,
			"on_addFileExcPattern_clicked" : self.addFileExc,
			"on_checkHiddenExcl_toggled" : self.excludeHidden,
			"on_checkBakFilesExcl_toggled" : self.excludeBak,
			"on_checkTempFilesExcl_toggled" : self.excludeTemp,
			"on_check~FilesExcl_toggled" : self.excludeTildeFiles,
			"on_maxDiskCheck_toggled" :self.maxDiskCheck,
			"on_everyCheck_toggled" : self.everyCheck,
			"on_batteryCheck_toggled" : self.batteryCheck,
			"on_cpuCheck_toggled" : self.cpuCheck,
			"on_deleteIfOlderCheck_toggled" : self.deleteIfOlderCheck, 
			"on_deleteIfSpaceCheck_toggled" : self.deleteIfSpaceCheck
		}
		if self.__sizeValue == -1:
			self.wTree.get_widget('maxDiskCheck').set_active(False)
			self.wTree.get_widget('sizeOptSpin').set_sensitive(False)
		else:
			self.wTree.get_widget('maxDiskCheck').set_active(True)

		if self.__modBackEveryVal == "-1":
			self.wTree.get_widget('everyCheck').set_active(False)
			self.wTree.get_widget('comboEvery').set_sensitive(False)
		else:
			self.wTree.get_widget('everyCheck').set_active(True)

		if str(self.__batteryLimitValue) == "-1":
			self.wTree.get_widget('batteryCheck').set_active(False)
			self.wTree.get_widget('spinBattery').set_sensitive(False)
		else:
			self.wTree.get_widget('batteryCheck').set_active(True)

		if str(self.__cpuLimitValue) == "-1":
			self.wTree.get_widget('cpuCheck').set_active(False)
			self.wTree.get_widget('spinCPU').set_sensitive(False)
		else:
			self.wTree.get_widget('cpuCheck').set_active(True)

		if str(self.__timeLimit) == "-1":
			self.wTree.get_widget('deleteIfOlderCheck').set_active(False)
			self.wTree.get_widget('spinTimeExceed').set_sensitive(False)
		else:
			self.wTree.get_widget('deleteIfOlderCheck').set_active(True)

		if str(self.__spaceLimit) == "-1":
			self.wTree.get_widget('deleteIfSpaceCheck').set_active(False)
			self.wTree.get_widget('spinSpaceLimit').set_sensitive(False)
		else:
			self.wTree.get_widget('deleteIfSpaceCheck').set_active(True)
		self.wTree.signal_autoconnect(dic)	
		# @var window : Contains the window object obtained by executing the wTree.get_widget() method.
		self.window=self.wTree.get_widget('Preferences')



	def everyCheck(self,widget):
		if self.wTree.get_widget('everyCheck').get_active()==False:
			self.__modBackEveryVal="-1"			
			self.wTree.get_widget('comboEvery').set_sensitive(False)
		else:
			self.wTree.get_widget('comboEvery').set_sensitive(True)


	def maxDiskCheck(self,widget):
		if self.wTree.get_widget('maxDiskCheck').get_active()==False:
			self.__sizeValue = -1			
			self.wTree.get_widget('sizeOptSpin').set_value(float(self.__sizeValue))
			self.wTree.get_widget('sizeOptSpin').set_sensitive(False)
		else:
			self.wTree.get_widget('sizeOptSpin').set_sensitive(True)

	def batteryCheck(self,widget):
		if self.wTree.get_widget('batteryCheck').get_active()==False:
			self.__batteryLimitValue = -1			
			self.wTree.get_widget('spinBattery').set_value(float(self.__batteryLimitValue))
			self.wTree.get_widget('spinBattery').set_sensitive(False)
		else:
			self.wTree.get_widget('spinBattery').set_sensitive(True)
	
	def cpuCheck(self,widget):
		if self.wTree.get_widget('cpuCheck').get_active()==False:
			self.__cpuLimitValue = -1			
			self.wTree.get_widget('spinCPU').set_value(float(self.__cpuLimitValue))
			self.wTree.get_widget('spinCPU').set_sensitive(False)
		else:
			self.wTree.get_widget('spinCPU').set_sensitive(True)

	def deleteIfOlderCheck(self,widget):
		if self.wTree.get_widget('deleteIfOlderCheck').get_active()==False:
			self.__timeLimit = -1			
			self.wTree.get_widget('spinTimeExceed').set_value(float(self.__timeLimit))
			self.wTree.get_widget('spinTimeExceed').set_sensitive(False)
		else:
			self.wTree.get_widget('spinTimeExceed').set_sensitive(True)

	def deleteIfSpaceCheck(self,widget):
		if self.wTree.get_widget('deleteIfSpaceCheck').get_active()==False:
			self.__spaceLimit = -1			
			self.wTree.get_widget('spinSpaceLimit').set_value(float(self.__spaceLimit))
			self.wTree.get_widget('spinSpaceLimit').set_sensitive(False)
		else:
			self.wTree.get_widget('spinSpaceLimit').set_sensitive(True)


	##@ brief Cancels the changes of the preferences and close the window.
	#
	# @param self The object pointer.
	# @param widget The type of the window.	
	# @author Roberto Cano Fernandez
	def cancel(self,widget):
		self.window.destroy()


	## @brief Saves the preferences choosen in this menu and closes the window
	#
	# @param self The object pointer.
	# @param widget The type of the window.		
	# @return writes the preferences into a file called "prueba.cfg"
	# @author Roberto Cano Fernandez
	def accept(self,widget):
		self.savePreferences(widget)
		self.window.destroy()


	## @brief Saves the preferences choosen in this menu
	#
	# @param self The object pointer.
	# @param widget The type of the window.		
	# @return writes the preferences into a file called "prueba.cfg"
	# @author Roberto Cano Fernandez
	def applyB(self,widget):
		self.savePreferences(widget)


	## @brief Saves the preferences choosen in this menu
	#
	# @param self The object pointer.
	# @param widget The type of the window.		
	# @return writes the preferences into a file called "prueba.cfg"
	# @author Roberto Cano Fernandez
	def savePreferences(self,widget):
		self.__re_blacklist = ""
		self.__re_whitelist = ""
		if not self.__proxy.hasSection("General"):		
			self.__proxy.addSection("General")
		if (str(self.__nautInt))=="0":
			self.__nautInt="False"
		if (str(self.__nautInt))=="1":
			self.__nautInt="True"
		self.__proxy.setNautilusIntegration(str(self.__nautInt))
		#self.__proxy.setLanguage(self.__lang)
		if not self.__proxy.hasSection("BackupInterval"):
			self.__proxy.addSection("BackupInterval")
		if (str(self.__modBackChange))=="0":
			self.__modBackChange="False"
		if (str(self.__modBackChange))=="1":
			self.__modBackChange="True"
		self.__proxy.setEveryChange(str(self.__modBackChange))
		self.__proxy.setPeriodically(self.__modBackEveryVal)
		self.__proxy.setDeleteIfOlder(str(self.__timeLimit))
		if not self.__proxy.hasSection("Lists"):		
			self.__proxy.addSection("Lists")
		if (self.__incHidden == "True"):
			self.__re_whitelist = self.__re_whitelist + "*/.*;"
		if (self.__incCnf == "True"):
			self.__re_whitelist=self.__re_whitelist + "*.cnf;"
		if (self.__incConf == "True"):
			self.__re_whitelist=self.__re_whitelist + "*.conf;"
		if (self.__excHidden == "True"):
			self.__re_blacklist=self.__re_blacklist + "*/.*;"
		if (self.__excBak == "True"):
			self.__re_blacklist=self.__re_blacklist + "*.bak;"
		if (self.__excTemp == "True"):
			self.__re_blacklist=self.__re_blacklist + "*.tmp;"
		if (self.__excTilde == "True"):
			self.__re_blacklist=self.__re_blacklist + "*~;"
		self.__re_whitelist = self.__re_whitelist[:len(self.__re_whitelist)-1]
		self.__re_blacklist = self.__re_blacklist[:len(self.__re_blacklist)-1]
		#esto separa los elementos de una lista mediante ; que es como deberia hacerse lo de los patrones, metiendolos en una lista y luego con la funcion .join
		self.__whitelist =";".join(self.__itemsAdded[:len(self.__itemsAdded)])
		self.__blacklist =";".join(self.__itemsExcluded[:len(self.__itemsExcluded)])
		self.__proxy.setFileLists(self.__whitelist,self.__re_whitelist,self.__blacklist,self.__re_blacklist)
		if not self.__proxy.hasSection("HDUsage"):		
			self.__proxy.addSection("HDUsage")
		self.__proxy.setBackupDirectory(self.__chooseDir)
		self.__proxy.setDeleteIfSpace(str(self.__spaceLimit))
		if not self.__proxy.hasSection("Limits"):		
			self.__proxy.addSection("Limits")
		#FIXME preguntar, pero me parece... que -1 en limitvalue es k no hay...
		self.__proxy.setBatteryChargeLimit(str(self.__batteryLimitValue))
		self.__proxy.setCpuUseLimit(str(self.__cpuLimitValue))
		self.__proxy.setMaxBackupSize(str(self.__sizeValue))
		#if a section doesn't exists, it's created.		
		# Flag para que no duplique la escritura del tree view
		self.__toUpdate = True

	## @brief Destroy method of the window
	#
	# @param self The object pointer.
	# @param widget The type of the window.
	# @author Roberto Cano Fernandez
	def DestroyWind(self,widget):
		self.window.destroy()


	## @brief Takes the value of spin button to determinate the maximun size of a file
	#
	# This method takes the value in the preferences window and
	# determinates the maximun size of a file.
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def setSizeValue(self,widget):	
		self.__sizeValue=self.wTree.get_widget('sizeOptSpin').get_text()
	

	## @brief Takes the value of check button to determinate integration with nautilus browser
	#
	# This method takes the value in the preferences window and
	# determinates if the nautilus integration is enabled or disabled, possible values are True, False(0)
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def actNautInt (self,widget):
		self.__nautInt=(self.wTree.get_widget('checkActNaut').get_active())


	## @brief Takes the value of check button to determinate the maximun size of a file
	#
	# This method takes the value in the preferences window and
	# determinates if the files are watched when changes are found in them
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def modBackIntChan (self,widget):
		self.__modBackChange=str(self.wTree.get_widget('checkModChange').get_active())





	## @brief Takes the value of spin button to determinate how long is the period of time
	#
	# This method takes the value in the preferences window and
	# determinates the period of time for watching files
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def setBackIntEvery (self,widget):
		#TODO??? case de los minutos y tal
		self.__modBackEveryVal=self.wTree.get_widget('comboEvery').get_active_text()


	## @brief Takes the value of directory button to determinate how long is the period of time
	# 
	# This method takes the value in the preferences window and
	# determinates the period of time for watching files
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def setChoosDir (self,widget):
		self.__chooseDir=self.wTree.get_widget('chooserDirectory').get_current_folder()


	## @brief Takes the value of combo button to determinate what is the language
	#
	# This method takes the value in the preferences window and
	# determinates the language of HD Lorean
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	#def setLang (self,widget):
		#self.__lang=self.wTree.get_widget('comboLanguage').get_active_text()


	## @brief Takes the value of check button to determinate if backups will automatically be deleted
	#
	# This method takes the value in the preferences window and
	# determinates if the backups greater than a max size will be
	# deleted
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def spaceExceed (self,widget):
		self.__spaceExceeded=str(self.wTree.get_widget('spinSpaceExceed').get_active_text())


	## @brief Takes the value of spin button to determinate the max size of backups to be deleted
	#
	# This method takes the value in the preferences window and
	# determinates the maximun size of the backups that users permits
	# the backups greater than this size will be deleted
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def setSpaceLimit (self,widget):
		self.__spaceLimit=self.wTree.get_widget('spinSpaceLimit').get_text()



	## @brief Takes the value of spin button to determinate the max time of backups to be deleted
	#
	# This method takes the value in the preferences window and
	# determinates the maximun time that a backup can stay in PC without
	# be deleted.
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def setTimeLimit (self,widget):
		self.__timeLimit=self.wTree.get_widget('spinTimeExceed').get_text()


	## @brief Takes the value of spin button to determinate the minimun change of battery
	#
	# This method takes the value in the preferences window and
	# determinates the minimun charge of battery neccesary to make backups
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def setBatteryLimit (self,widget):
		self.__batteryLimitValue=self.wTree.get_widget('spinBattery').get_text()


	## @brief Takes the value of spin button to determinate the max CPU percent use
	#
	# This method takes the value in the preferences window and
	# determinates the maximun CPU percent use that permits make backups
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def setCpuLimit (self,widget):
		self.__cpuLimitValue=self.wTree.get_widget('spinCPU').get_text()


	## @brief Searches in a given list a given string
	#
	#
	# @param self the object pointer
	# @param lis given list to search in
	# @param st given string to find
	# @author Roberto Cano
	def searchInList(self,lis,st):			
		aux=lis.split(";")
		i=0
		find=False
		if aux[0]!= "":
			while ((i<(len(aux)))and (not find)):
				find=aux[i]==st			
				i=i+1
		return find


	## @brief Includes in the treeview a file url
	#
	# This method shows int the treeview an included url from a file chooser
	# dialog. That file will be watched
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Carlos Agra- Roberto Cano
	def addFileIncButton (self,widget):
		selectedFile = self.__fileSelect("Please select a file to backup")
		if not selectedFile:
			return
		find=selectedFile in self.__itemsAdded
		if not find:
			self.__itemsAdded.append(selectedFile)
			itt = self.__liststoreInclude.append([selectedFile])
		

	## @brief Includes in the treeview a folder url
	#
	# This method shows in the tree view an included url from a choose folder
	# dialog. This folder will be watched
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Carlos Agra-Roberto Cano
	def addFolderIncButton (self,widget):
		directory = self.__directorySelect("Please select a directory to back up")
		if not directory:
			return
		find = directory in self.__itemsAdded
		if not find:
			self.__itemsAdded.append(directory)		
			itt = self.__liststoreInclude.append([directory])


	## @brief Removes in the treeview a folder url
	#
	# This method removes in the tree view an removed url folder
	# selected.
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Carlos Agra-Roberto Cano
	def removeIncButton (self,widget):
		try:
			model,itt = self.__treeviewInclude.get_selection().get_selected()
			selection = self.__liststoreInclude.get_value(itt, 0)
			self.__liststoreInclude.remove(itt)
			# Remove the directory from itemsExcluded
			self.__itemsAdded.remove(selection)
		except:
			#TODO algo mejor que esto->print "There is no folder selected"
			pass

	## @brief Includes in the treeview a file pattern to include
	#
	# This method includes in the tree view a file pattern
	# selected.
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Carlos Agra-Roberto Cano
	def addFileInc (self,widget):
		##llamamos a la ventana nueva
		app=AddFileIncPattern.AddFileIncPattern(self.__includePattern)
		#Aqui devolvemos la cadena del patron. 
		self.__includePattern=AddFileIncPatern.addPattern(widget)
		#print "patron incluido preferencias: " + self.__includePattern
		#FIXME esto no tira
	## @brief Includes in the exclude treeview a file url
	#
	# This method shows in the treeview an excluded url from a file chooser
	# dialog. This file will be excluded from watching
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Carlos Agra- Roberto Cano
	def addFileExcButton (self,widget):
		selectedFile = self.__fileSelect('Please select a file to exclude')
		if not selectedFile:
			return
		find = selectedFile in self.__itemsExcluded
		if not find:
			self.__itemsExcluded.append(selectedFile)
			itt = self.__liststoreExclude.append([selectedFile])


	## @brief Includes in the exclude treeview a folder url
	#
	# This method shows in the tree view an excluded url from a choose folder
	# dialog. This folder will be excluded from watching
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Carlos Agra-Roberto Cano
	def addFolderExcButton (self,widget):
		directory = self.__directorySelect('Please select a directory to exclude')
		if not directory:
			return
		find = directory in self._itemsExcluded
		if not find:
			self.__itemsExcluded.append(directory)		
			itt = self.__liststoreExclude.append([directory])		

	
	## @brief Removes in the treeview a folder url
	#
	# This method removes in the tree view an removed url folder
	# selected.
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Carlos Agra-Roberto Cano
	def removeExcButton (self,widget):
		try:
			model,itt = self.__treeviewExclude.get_selection().get_selected()
			selection = self.__liststoreExclude.get_value(itt, 0)
			self.__liststoreExclude.remove(itt)
			# Remove the directory from itemsExcluded
			self.__itemsExcluded.remove(selection)
		except:
			#TODO señalizacion mejor que esto ->print "There is no folder selected"
			pass


	## @brief Includes in the treeview a file pattern to include
	#
	# This method includes in the tree view a file pattern
	# selected.
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Carlos Agra-Roberto Cano
	def addFileExc (self,widget):
		##TODO este metodo
		pass

	## @brief Takes the value of check button to determinate if hidden files will be watched
	#
	# This method takes the value in the preferences window and
	# determinates if hidden files will be watched 
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def includeHidden (self,widget):
		self.__incHidden=str(self.wTree.get_widget('checkHidden').get_active())


	## @brief Takes the value of check button to determinate if bak files will be watched
	#
	# This method takes the value in the preferences window and
	# determinates if bak files will be watched 
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def includeCnf (self,widget):
		self.__incCnf=str(self.wTree.get_widget('checkCnfFiles').get_active())


	## @brief Takes the value of check button to determinate if *.temp files will be watched
	#
	# This method takes the value in the preferences window and
	# determinates if *.temp files will be watched
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def includeConf (self,widget):
		self.__incConf=str(self.wTree.get_widget('checkConfFiles').get_active())


	


	## @brief Takes the value of check button to determinate if hidden files will be excluded
	#
	# This method takes the value in the preferences window and
	# determinates if hidden files will be excluded to be watching
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def excludeHidden (self,widget):
		self.__excHidden=str(self.wTree.get_widget('checkHiddenExcl').get_active())


	## @brief Takes the value of check button to determinate if bak files will be excluded
	#
	# This method takes the value in the preferences window and
	# determinates if bak files will be excluded to be watching
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def excludeBak (self,widget):
		self.__excBak=str(self.wTree.get_widget('checkBakFilesExcl').get_active())


	## @brief Takes the value of spin button to determinate if *.temp files will be excluded
	#
	# This method takes the value in the preferences window and
	# determinates if *.temp files will be exclude to be watched
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def excludeTemp (self,widget):
		self.__excTemp=str(self.wTree.get_widget('checkTempFilesExcl').get_active())


	## @brief Takes the value of check button to determinate if temporal files will be excluded
	#
	# This method takes the value in the preferences window and
	# determinates if temporal files will be excluded to be watched
	# @param self the object pointer
	# @param widget the widget for take the value
	# @author Roberto Cano Fernández
	def excludeTildeFiles (self,widget):
		self.__excTilde=str(self.wTree.get_widget('checkTildeFilesExcl').get_active())

	## @brief Method that allows user to select a directory with a file chooser
	#
	# @author Carlos Agra
	def __directorySelect(self,title="Please select a directory"):
		dialog = gtk.FileChooserDialog(title, None, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, 
			(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		dialog.set_current_folder(os.environ['HOME'])
	
		if dialog.run() == gtk.RESPONSE_OK:
			directory = dialog.get_filename()
		else:
			directory = None

		dialog.destroy()
		return directory

	## @brief Method that allows user to select a file with a file chooser
	#
	# @author Carlos Agra
	def __fileSelect(self,title='Please select a single file'):
		dialog = gtk.FileChooserDialog(title, None, gtk.FILE_CHOOSER_ACTION_OPEN, 
			(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		dialog.set_current_folder(os.environ['HOME'])
	
		if dialog.run() == gtk.RESPONSE_OK:
			selectedFile = dialog.get_filename()
		else:
			selectedFile = None

		dialog.destroy()
		return selectedFile


	

