# -*- coding: utf-8 -*-

#Copyright (C) 2008 Federico Mon Trotti, Daniel Tabas, Carlos Agra, Roberto Cano

#This file is part of HDLorean.
#
#HDLorean is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 2 of the License.

#HDLorean is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with HDLorean.  If not, see <http://www.gnu.org/licenses/>.

import pygtk
pygtk.require('2.0')
import gtk, gobject


class ProgressBar:
	__vbox=gtk.VBox(False, 5)
	
	# @brief Adds a new progress bar to the window
	#
	# this method creates a new alignment and a progress bar inside it
	# @param self
	# @param widget
	# @param opName string with the name of the operation in progress
	# @return pbar the new progress bar
	# @return align the new alignment
	# @author Roberto Cano
	def addBar(self,widget,opName):
		self.__pBarCont=self.__pBarCont+1
		pbar = gtk.ProgressBar()
		# Create a centering alignment object
		align = gtk.Alignment(0.5, 0.5, 0, 0)
		self.__vbox.pack_start(align, False, False, 5)
		align.show()
		align.add(pbar)
		pbar.show()
		pbar.set_text(str(opName))
		return pbar,align
		
	# @brief removes a selected progress bar from the window
	#
	# @param self
	# @param widget
	# @param barId id of the progress bar to be removed
	# @param alignId id of the alignment where is placed the pBar
	# @author Roberto Cano
	def remBar(self,widget,barId,alignId):
		barId.destroy()
		alignId.destroy()
	# @brief updates the value of a selected progress bar from the window
	#
	# @param self
	# @param widget
	# @param barId id of progress bar to update
	# @param perc new value of the progress bar (0-100)
	# @author Roberto Cano
	def updateProg(self,widget,barId,perc):
		aux=float(perc)/100		
		barId.set_fraction(aux)
	
	
	# Clean up allocated memory
	#
	# @author http://www.pygtk.org/ modified by Roberto Cano
	def destroy_progress(self, widget, data=None):
		gtk.main_quit()
	# @author http://www.pygtk.org/, modified by Roberto Cano
	def __init__(self):
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_resizable(True)
	
		self.window.connect("destroy", self.destroy_progress)
		self.window.set_title("Snapshot progress")
		self.window.set_border_width(0)
	
		#vbox = gtk.VBox(False, 5)
		self.__vbox.set_border_width(10)
		self.window.add(self.__vbox)
		self.__vbox.show()
		
		separator = gtk.HSeparator()
		self.__vbox.pack_start(separator, False, False, 0)
		separator.show()
	
		# rows, columns, homogeneous
		table = gtk.Table(2, 2, False)
		self.__vbox.pack_start(table, False, True, 0)
		table.show()
	
		# Add a button to exit the program
		button = gtk.Button("close")
		button.connect("clicked", self.destroy_progress)
		self.__vbox.pack_start(button, False, False, 0)

		# This makes it so the button is the default.
		button.set_flags(gtk.CAN_DEFAULT)
	
		# This grabs this button to be the default button. Simply hitting
		# the "Enter" key will cause this button to activate.
		button.grab_default ()
		button.show()
	
		self.window.show()
	


if __name__ == "__main__":
		app=ProgressBar()
		gtk.main()
