# -*- coding: utf-8 -*-

#Copyright (C) 2008 David Molina, Salvador de la Puente, Federico Mon Trotti

#This file is part of HDLorean.
#
#HDLorean is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#HDLorean is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with HDLorean.  If not, see <http://www.gnu.org/licenses/>.


from DataBaseWrapper import *
import time
from hdlogger import HDLogger


##@brief
#
#XDelta3WrapperDB database interfaze class
#
#This class implements all the functions needed 
#to control the XDelta3WrapperDB database of HdLorean.
class XDelta3WrapperDB:
	
	__DBW = None #<Contains the DataBaseWrapper.
	__logger = None #<Contains the logger.
	__log =None #<Lets write logs messages.
	__keys = None

	
	##@brief
	#
	#The constructor of the class.	
	def __init__(self):
		self.__DBW =  DataBaseWrapper()
		self.__logger = HDLogger.HDLogger()		
		self.__keys = None
		self.__log = self.__logger.setupHDLogger("hdlorean.db.XDelta3Wrapper")

		


	##@brief 
	#
	#Create the table needed to the XDelta3Wrapper database.
	def createTable(self):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				table = 'CREATE TABLE IF NOT EXISTS xdelta3wrapperdb (' 
				table = table + 'id INTEGER PRIMARY KEY AUTOINCREMENT, '
				table = table + 'timestamp TIMESTAMP NOT NULL, '
				table = table + 'chtype VARCHAR(255) NOT NULL, '
				table = table + 'btype VARCHAR(255) NOT NULL, ' 
				table = table + 'GMT INTEGER NOT NULL, '
				table = table + 'DST INTEGER NOT NULL, '
				table = table + 'status VARCHAR(255) NOT NULL, '
				table = table + 'size INTEGER NOT NULL, '
				table = table + 'location VARCHAR(255) NOT NULL, '
				table = table + 'initpath VARCHAR(255) NOT NULL, '
				table = table + 'finalpath VARCHAR(255) NOT NULL,'
				table = table +	'requestor	VARCHAR(255) NOT NULL, '
				table = table +	'parent	INTEGER NOT NULL); '
				index = 'CREATE INDEX IF NOT EXISTS indice ON xdelta3wrapperdb(timestamp) '
				self.__DBW.createTable(table,index)
				self.__log.info('XDelta3WrapperDB database created without problems.')   
				self.__keys = set(['id','timestamp','chtype','btype','GMT',
					'DST','status','size','location','initpath',
					'finalpath','requestor','parent'])
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s, %s. Error: %s.', table, index, e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()
	
		
	##@brief 
	#
	#Inserts in the XDelta3WrapperDB table a new row with the values of the parameters.
	#
	#@param timestamp String with a date in unix format. 
	#@param chtype String with one of the next words: MODIFY, MOVE, DELETE, CREATE, CRON.
	#@param btype String that indicates de type of backup: 'INT' or 'DIFF'.
	#@param GMT String with an integer number.
	#@param DST String with an integer number: '0'(Winter) or '1'(Summer).
	#@param status String with the status of the backup: FIXME
	#@param size String with the size of the backup.
	#@param location String with the location of the backup: FIXME
	#@param init String with the first path of the file.
	#@param final String with the actual path of the file.
	#@param req String with who ordered the change.
	#@param parent String with who ordered the change.
	def insert(self,timestamp,chtype,btype,GMT,DST,status,size,location,init,final,req,parent=0):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				insert = "insert into xdelta3wrapperdb (timestamp,chtype,btype,GMT,DST,status,size,location,initpath,finalpath,requestor,parent) values (?,?,?,?,?,?,?,?,?,?,?,?)"
				argv = (timestamp,chtype,btype,GMT,DST,status,size,location,init,final,req,parent)
				#query = "insert into xdelta3wrapperdb (timestamp,chtype,btype,GMT,DST,status,size,location,initpath,finalpath,requestor)"	
				#query = query + " values ("+timestamp+",'"+chtype+"','"+btype+"',"+GMT+","+DST+",'"+status+"',"+size+",'"+location+"','"+init+"','"+final+"','"+req+"');"
				self.__DBW.noResultQuery(insert,argv)
			except IntegrityError, e:
				self.__log.critical('Integrity error has happened in the database.')
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.', query, e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()
	
	
	##@brief
	#
	#!!!!!!!!!!!! D E P R E C A T E D !!!!!!!!!!!!
	#Extracts from the table the information requested.
	#
	#Create a select query from the input data and returns the results in an array.
	#
	#@param listcolums column List to draw from the table. Example: ['timestamp','wrote']
	#@param listcond Optional. List of conditions. Example: ['timestamp=02.03.07','and','wrote=0', "chtype='MODIFY'"]
	#@param listord Optional. List of orders. Example: ['timestamp ASC','wrote']  
	#@param listgroup Optional. List of groups. Example: ['wrote']
	#@param listhav  Optional. List of having.
	#
	#@return An array with the selected tuples
	def select(self,listcolumns,listcond=[],listord=[],listgroup=[],listhav=[]):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				select ='select ' #< keeps the sql query
				columns = '' #< keeps the columsn in a string		   
				i=0  
				# Loop that links the list in a string			  
				for x in listcolumns:
					if i==len(listcolumns)-1:
						columns = columns + x
					else:
						columns = columns + x + ", "
					i=i+1 
				select = select + columns + " from xdelta3wrapperdb"
				if len(listcond)>0:
					select = select + " where "
					condition='' #< keeps conditions in a string
					i=0
					# Loop that links the list in a string
					for x in listcond:				 
						if i==len(listcond)-1:
							condition = condition + x
						else:
							condition = condition + x + " "
						i=i+1					
					select = select + condition
				if len(listord)>0:
					select = select + " order by "
					order ='' #< keeps order in a string
					i=0
					# Loop that links the list in a string
					for x in listord:				 
						if i==len(listord)-1:
							order = order + x
						else:
							order = order + x + ", "
						i=i+1
					select = select + order
				if len(listgroup)>0:
					select = select + " group by "
					group = '' #< keeps groups in a string
					i=0
					# Loop that links the list in a string
					for x in listgroup:
						if i==len(listgroup)-1:
							group = group + x
						else:
							group = group + x + ", "
						i=i+1
					select = select + group
				if len(listhav)>0:
					select = select + " having "
					having='' #< keeps conditions in a string
					i=0
					# Loop that links the list in a string
					for x in listhav:
						if i==len(listhav)-1:
							having = having + x
						else:
							having = having + x + ", "
						i=i+1
					select = select + having
				select = select +";"
				result = self.__DBW.resultQuery(select,())
				return result 
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.', select, e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()
		  
	
	##@brief
	#
	#Update the timestamp of the path with the old_timestamp
	#
	#@param path String with the path of the file.
	#@param old_timestamp The timestamp in the database
	#@param new_timestamp The new timestamp.
	def updateTimestamp(self,path,old_timestamp,new_timestamp):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				update = 'update xdelta3wrapperdb set timestamp=? where finalpath=? and timestamp=?;'
				argv = (new_timestamp,path,old_timestamp)
				self.__DBW.noResultQuery(update,argv)				
			except IntegrityError, e:
				self.__log.critical('Integrity error has happened in the database.')
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.', update, e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()




	##@brief Function to delete a version of a file (a backup of a file)
	#
	#@param path: path of the file
	#@param timestamp: timestamp of the version of the file to be deleted
	def deleteBackups(self, path, init_timestamp=0, final_timestamp=0):
		if not final_timestamp: final_timestamp = init_timestamp

		try:
			next_timestamp = 0
			list_to_move = []
			list_to_delete = []
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				slashpath = path+"/%%"
				# Desmarcar prohibidos
				update = "update xdelta3wrapperdb set status='NONE' where timestamp <= ? and timestamp >= ? and (finalpath=? or finalpath like ?) and btype='INT';"
				argv = (str(final_timestamp),str(init_timestamp),path,slashpath)
				self.__DBW.noResultQuery(update,argv)
		
				# Buscar siguiente
				#next_timestamp = self.__db.nextTimestamp(path,final_timestamp) #TODO:Comprobar que esto nos da lo que queremos
				select_next_timestamp = "select timestamp from xdelta3wrapperdb where timestamp > ? and (finalpath=? or finalpath like ?) order by timestamp ASC limit 1;"
				argv = (str(final_timestamp),path,slashpath)
				next_timestamp = self.__DBW.resultQuery(select_next_timestamp,argv)

				if next_timestamp:
					next_timestamp = next_timestamp[0][0]

					# Buscar los que tengan timestamp siguiente
					substract = "select finalpath from xdelta3wrapperdb where timestamp = ?"
					# Buscar los ultimos de cada archivo en el intervalo, y no tienen versión en el timestamp siguiente.
					select = "select id,finalpath,max(timestamp),GMT,DST,btype from xdelta3wrapperdb where timestamp <= ? and timestamp >= ? and (finalpath=? or finalpath like ?) and finalpath not in ("+substract+") group by finalpath;"
					argv = (str(final_timestamp),str(init_timestamp),slashpath,str(next_timestamp))
					toPromote = self.__DBW.resultQuery(select,argv)
					if toPromote:
						list_to_move = [x[1:] for x in toPromote]
						for entry in toPromote:
							update = "update xdelta3wrapperdb set timestamp=? where id = ?;"
							argv = (str(next_timestamp),str(entry[0]))
							self.__DBW.noResultQuery(update,argv)
			
					# Para todos los de timestamp siguiente marcar como prohibido su integral (si tiene)
					parents = "select parent from xdelta3wrapperdb where timestamp = ? and (finalpath=? or finalpath like ?)"
					update = "update xdelta3wrapperdb set status = 'forbidden' where id in ("+parents+");"
					argv = (str(next_timestamp),path,slashpath)
					self.__DBW.noResultQuery(update,argv)
				# Borrar (y añadir a lista) todos los no prohibidos
				select = "select finalpath,timestamp,GMT,DST,btype from xdelta3wrapperdb where status != 'forbidden' and timestamp <= ? and timestamp >= ? and (finalpath=? or finalpath like ?);"
				argv = (str(final_timestamp),str(init_timestamp),path,slashpath)
				list_to_delete = self.__DBW.resultQuery(select,argv)
				delete = "delete from xdelta3wrapperdb where status != 'forbidden' and timestamp <= ? and timestamp >= ? and (finalpath=? or finalpath like ?);"
				argv = (str(final_timestamp),str(init_timestamp),path,slashpath)
				self.__DBW.noResultQuery(delete,argv)
			except IntegrityError, e:
				self.__log.info('Integrity error has happened in the database.')
			except SQLError, e:
				self.__log.info('SQL error has happened in the database: %s. Error: %s.', update, e)
			except ConnError, e:
				self.__log.info('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()
			return next_timestamp, list_to_move, list_to_delete
			
	##@brief
	#
	#

#	def deleteBackups(self, path, init_timestamp=None, final_timestamp=None):
#		result = (0,0,0)
#		try:
#			self.__DBW.openCon('/HdXDelta3WrapperDB')	
#			try:
#				backupsToMove = []
#				backupsToDelete = []
#				#For each path
#
#				#TODO: if final_timestamp > init_timestamp: #lanzar excepcion
#
#				if not final_timestamp and not init_timestamp:
#					select = 'select * from xdelta3wrapperdb where (finalpath ="'+path+'" or finalpath like "'+path+'/%");'
#					delete = 'delete from xdelta3wrapperdb where (finalpath ="'+path+'" or finalpath like "'+path+'/%");'
#					toDelete = self.__DBW.resultQuery(select)
#					self.__DBW.noResultQuery(delete)
#					if toDelete:
#						backupsToDelete += toDelete
#					result = (0,backupsToMove,backupsToDelete)
#					return result
#
#				else:
#					if init_timestamp and not final_timestamp:
#						final_timestamp = init_timestamp
#
#
#					toMove = []
#					toDelete = []
#				
#					select_next_timestamp = 'select timestamp from xdelta3wrapperdb where timestamp > '+str(init_timestamp)+' and (finalpath ="'+path+'" or finalpath like "'+path+'/%") order by timestamp ASC limit 1;'
#					next_timestamp = self.__DBW.resultQuery(select_next_timestamp)
#
#					if next_timestamp:
#						next_timestamp = next_timestamp[0][0]
#						select = 'select * from xdelta3wrapperdb where timestamp = '+str(next_timestamp)+' and (finalpath ="'+path+'" or finalpath like "'+path+'/%") group by finalpath order by timestamp DESC;'
#						has_next_backup = self.__DBW.resultQuery(select)
#						if has_next_backup:
#							for latest_backup in has_next_backup:
#								
#								update = 'update xdelta3wrapperdb set status=NONE where timestamp <= '+str(final_timestamp)+' and timestamp >='+str(init_timestamp)+' and finalpath="'+latest_backup[-1]+'" and btype="INT";'
#								self.__DBW.noResultQuery(update)
#								
#								if latest_backup[3] == 'DIFF':
#									select = 'select id from xdelta3wrapperdb where timestamp <= '+str(final_timestamp)+' and timestamp >= '+str(init_timestamp)+' and finalpath="'+latest_backup[-1]+'" and btype="INT" order by timestamp DESC limit 1;'
#									lastIntIDs = self.__DBW.resultQuery(select)
#									if lastIntIDs:
#										lastIntID = lastIntIDs[0][0]
#										update = "update xdelta3wrapperdb set status='forbidden' where id="+str(lastIntID)+';'
#										self.__DBW.noResultQuery(update)
#										if len(lastIntIDs)>1:
#											prevLastIntID = lastIntIDs[1]
#											delete = "delete from xdelta3wrapperdb where id ="+str(prevLastIntID)+'and ;'
#											self.__DBW.noResultQuery(delete)
#
#											
#									#TODO: En algun momento hay que mirar si nos quedan forbiddens anteriores, no?
#						select = 'select * from xdelta3wrapperdb where timestamp >= '+str(init_timestamp)+' and timestamp <'+str(next_timestamp)+ ' and (finalpath ="'+path+'" or finalpath like "'+path+'/%") group by finalpath order by timestamp DESC;'
#						nexts_backups = self.__DBW.resultQuery(select)			
#						def to_be_moved(a): return not a in has_next_backup
#						toMove = filter(to_be_moved, nexts_backups)
#						if toMove:
#							for backup in toMove:
#								update = "update xdelta3wrapperdb set timestamp="+str(next_timestamp)+" where finalpath='"+backup[-1]+"' and timestamp="+str(backup[1])+';'
#								self.__DBW.noResultQuery(update)
#
#						select = 'select * from xdelta3wrapperdb where timestamp >= '+str(init_timestamp)+' and timestamp <'+str(next_timestamp)+ ' and (finalpath ="'+path+'" or finalpath like "'+path+'/%") and not status like "forbidden" order by timestamp DESC;'
#						toDelete = self.__DBW.resultQuery(select)
#						delete = 'delete from xdelta3wrapperdb where timestamp >= '+str(init_timestamp)+' and timestamp <'+str(next_timestamp)+ ' and (finalpath ="'+path+'" or finalpath like "'+path+'/%") and not status like "forbidden" order by timestamp DESC;'
#						self.__DBW.noResultQuery(delete)
#					else:
#						select = 'select * from xdelta3wrapperdb where timestamp >= '+str(init_timestamp)+' and (finalpath ="'+path+'" or finalpath like "'+path+'/%") order by timestamp DESC;'
#						toDelete = self.__DBW.resultQuery(select)
#						delete = 'delete from xdelta3wrapperdb where timestamp >= '+str(init_timestamp)+' and (finalpath ="'+path+'" or finalpath like "'+path+'/%");'
#						self.__DBW.noResultQuery(delete)
#					result = (next_timestamp,toMove,toDelete)
#					return result
#			except IntegrityError, e:
#				self.__log.critical('Integrity error has happened in the database.')
#			except SQLError, e:
#				self.__log.critical('SQL error has happened in the database: DeleteBackups. Error: %s.',e)
#			except ConnError, e:
#				self.__log.critical('The connection to the database doesnt exits.')
#		finally:
#			self.__DBW.closeCon()

	
	##@brief
	#
	#Search for the latest backup of a file.
	#
	#@param timestamp String with de date in unix format.
	#@param files List of strings, each string is the complete path of the file. Example: ['/home/Documents/example.py','/home/example2.txt']
	#@param Int Integer, 1 if you only want Int backups or 0 if you want the lastest backup.
	#@param recursive Optional Boolean, true if you want to do a recursive search.
	#
	#@return A list with the rows selects in the same order of paths in the list files, if a path has not result in that
	#position will be an empty sequence: ()
	def lastBackUp(self,timestamp,files,Int,recursive=False):
		list = []
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
				
			try:
				btype = ""
				if Int: btype="and btype='INT'"
				for path in files:
					operator = "finalpath='"+path+"'"
					if recursive: 
					#	operator = "(finalpath like '"+path+"' or finalpath like '"+path+"%' or finalpath like '"+path+"/%' )"
					#select = "select * from xdelta3wrapperdb where "+operator+" "+btype+" and timestamp<="+str(timestamp)+" order by timestamp DESC;"
					# =====
						slashpath = path+"/%%"
						select = "select * from xdelta3wrapperdb where (finalpath like ? or finalpath like ?% or finalpath like ?)"+btype+"  and timestamp <= ? order by timestamp DESC;"
						argv = (path,path,slashpath,str(timestamp))
					else:
						select = "select * from xdelta3wrapperdb where finalpath =? "+btype+" and timestamp<=? order by timestamp DESC;"
						argv = (path,str(timestamp))
					result = self.__DBW.resultQuery(select,argv)
					#self.__timeChange(result)
					if (result != None):
						list.append(result[0])
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.', select, e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()
		return list
	
	  
	##@brief
	#
	#Search for the all backups between the two params if there are not None.
	#If both params are None search in all table.
	#If oldest is None search for the rows before latest.
	#If latest is None search for the rows after oldest. 
	#
	#@param oldest String with the unix timestamp format.
	#@param latest String with the unix timestamp format.
	#
	#@return None if there is not result.
	#@return List of the rows selected.
	def getBetween(self,oldest,latest): 
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				argv = ()
				if (oldest==None and latest==None):
					select = "select distinct timestamp from xdelta3wrapperdb order by timestamp;"
				else:
					if (oldest!=None and latest!=None):
						select = "select distinct timestamp from xdelta3wrapperdb where timestamp>=? and timestamp<=? order by timestamp;"
						argv=(oldest,latest)
					else:
						if (oldest==None and latest!=None):
							select = "select distinct timestamp from xdelta3wrapperdb where timestamp<=? order by timestamp;"
							argv=(latest,)
						else:
							if (oldest!=None and latest==None):
								select = "select distinct timestamp from xdelta3wrapperdb where timestamp>=? order by timestamp;"
								argv=(oldest,)
				result = self.__DBW.resultQuery(select,argv)
				return result
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.',select,e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()

			
	##@brief
	#
	#Search for the oldest timestamp in the data base.
	#
	#@return 0 if there is not result.
	#@return The latest timestamp.
	def firstTimestamp(self):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				select = "select timestamp from xdelta3wrapperdb order by timestamp limit 1;"
				result = self.__DBW.resultQuery(select,());
				timeS = 0				
				if result!=None:
					timeS = int(result[0][0])
				return timeS		 
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.',select,e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()
			
	
	##@brief
	#
	#Search for the latest timestamp in the data base.
	#
	#@return 0 if there is not result.
	#@return The latest timestamp.
	def lastTimestamp(self):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				select = "select timestamp from xdelta3wrapperdb order by timestamp DESC limit 1;"
				result = self.__DBW.resultQuery(select,());
				timeS = 0		
				if result!=None:
					timeS = int(result[0][0])
				return timeS	
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.',select,e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()
		
	
	##@brief
	#
	#Search for the next timestamp of the param.
	#
	#@return 0 if there isnt result.
	#@return The next timestamp.
	def prevTimestamp(self,timestamp):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				select = "select timestamp from xdelta3wrapperdb where timestamp<? order by timestamp DESC limit 1;"
				argv = (str(timestamp),)
				result = self.__DBW.resultQuery(select,argv);
				timeS = 0
				if result != None:
					timeS = int(result[0][0])
				return timeS		 
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.',select,e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()
	

	##@brief
	#
	#Search for the previous timestamp of the param.
	#
	#@return None if there isnt result.
	#@return The next timestamp.
	def nextTimestamp(self,timestamp):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				select = "select timestamp from xdelta3wrapperdb where timestamp>? order by timestamp ASC limit 1;"
				argv = (str(timestamp),)
				result = self.__DBW.resultQuery(select,argv);
				timeS = 0
				if result != None:
					timeS = int(result[0][0])
				return timeS	 
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.',select,e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()  


	##@brief
	#
	#Search for the DST of the file with the timestamp in the param.
	#
	#@param timestamp String with the unix timestamp format.
	#
	#@return None if there is not result.
	#@return 1 if DST was in summer.
	#@return 0 if DST was in winter.
	def wasDST(self,timestamp):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				select = "select DST from xdelta3wrapperdb where timestamp=?;"
				argv = (timestamp,)
				result = self.__DBW.resultQuery(select,argv)
				if result!=None:
					DST = result[0]
					return DST[0]
				else:
					return None		 
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.',select,e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()
		
	
	##@brief
	#
	#Search for the GMT of the file with the timestamp in the param.
	#
	#@param timestamp String with the unix timestamp format.
	#
	#@return None if there isnt result.
	#@return integer to add to the GMT.
	def whichGMT(self,timestamp):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				select = "select GMT from xdelta3wrapperdb where timestamp=?;"
				argv=(timestamp,)
				result = self.__DBW.resultQuery(select,argv);
				if result != None:
					DST = result[0]
					return DST[0]
				else:
					return None			
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.', select, e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()
		
		
	##@brief
	#
	#Search for the snapshots before timestamp and like path.  
	#
	#@param timestamp String with the unix timestamp format.
	#@param path String whit the path of the file.
	#@param recursive Boolean used to select all files under the path(True).
	#@param latest Boolean used to select the backup with the timestamp in the first param(False),
	#or all backup with lower timestamp (True). 
	#@param btype Optional string with the type of backup to search.
	#
	#@return array with the rows selected.
	def getSnapshots(self,timestamp,path,recursive,latest,btype='all',grouped=''):
		result = None
		listArgs = []
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				if (not latest): timecondition='='
				else: timecondition='<='

				if (recursive):
						#if path == '': path = "/"
						slashpath = path+"/%%"
						comparison = "select * from xdelta3wrapperdb where timestamp "+timecondition+"? and (finalpath like ? or finalpath like ?)"
						listArgs.extend([timestamp,slashpath,path])
				else: 
						comparison = "select * from xdelta3wrapperdb where timestamp "+timecondition+"? and finalpath like ?"
						listArgs.extend([timestamp,path])
				
				groupp = " "
				#if grouped != '': groupp = " group by "+grouped
				if (btype!='all'):
					comparison = comparison + "btype=?"
					listArgs.append(btype)
					
					#select = "select * from xdelta3wrapperdb where timestamp"+timecondition + str(timestamp) +" and "+ comparison + " btype=" + btype + " " + groupp + " order by timestamp DESC;"

				if grouped != '' and grouped in self.__keys:
					comparison = comparison + " group by "+grouped
					
				select = comparison + " order by timestamp DESC;"
				result = self.__DBW.resultQuery(select,tuple(listArgs))
				return result
				#self.__timeChange(result)
			except IntegrityError, e:
				self.__log.critical('Integrity error has happened in the database.')
				raise IntegrityError, e   
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.', select, e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()
		
	
	##@brief
	#
	#Search for the snapshots of a path.  
	#
	#@param path String whit the path of the file.
	#@param recursive Boolean used to select all files under the path(True).
	#@param latest Boolean used to select the backup with the timestamp in the first param(False),
	#or all backup with lower timestamp (True). 
	#@param btype Optional string with the type of backup to search.
	#
	#@return array with the rows selected.
	def getAllSnapshots(self,path,recursive,latest,btype='all'):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				listArgs = []
				slashpath = path+"/%%"
				if (recursive): 
					comparison = "(finalpath like ? or finalpath like ?)"
					listArgs.extend([slashpath,path])
				else: 
					comparison = "finalpath like ?"
					listArgs = [path]

				if (btype!='all'):
					select = "select * from xdelta3wrapperdb where  "+ comparison + " btype=? order by timestamp DESC;"
					listArgs.append(btype)
				else:
					select = "select * from xdelta3wrapperdb where "+ comparison +" order by timestamp DESC;"
				result = self.__DBW.resultQuery(select,tuple(listArgs));
				return result				
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.', select, e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()
		
   
	##@brief
	#
	#Delete all snapshots with a path.  
	#
	#@param timestamp Unix time format.
	#@param location String with de backup location.
	#@param path String whit the path of the file.
	def deleteSnapshot(self,timestamp,location,path):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				delete = "delete from xdelta3wrapperdb where finalpath=? and location=? and timestamp=?;"
				argv = (path,location,timestamp)
				self.__DBW.noResultQuery(delete,argv)
			except IntegrityError, e:
				self.__log.critical('Integrity error has happened in the database.')
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.',delete,e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()
		
		
	##@brief
	#
	#Update de chtype info to forbidden  
	#
	#@param timestamp Unix time format.
	#@param path String whit the path of the file.
	#@param location String with the locantion of the backup.
	#@param btype String with the type of the backup.
	def markAsForbidden(self,timestamp,path,location=False,btype=False):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				update = "update xdelta3wrapperdb set status='forbidden' where finalpath=? and timestamp=?"
				if location: 
					update += " and location=?"
					argv = (path,timestamp,location)
				if btype: 
					update += " and btype=?"
					argv = (path,timestamp,btype)
				update += ";"
				self.__DBW.noResultQuery(update,argv)			   
			except IntegrityError, e:
				self.__log.critical('Integrity error has happened in the database.')
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.', update, e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()



	##@brief
	#
	#Search for the info of the backups of the path.
	#
	#@param path String with a path.
	#
	#@return An array with the number of backups,the date of the last backup,
	#the size of the file. In that order.
	def backUpInfo(self,path,recursive=False):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				slashpath = path+"/%%"
				if recursive:
					select = "select * from xdelta3wrapperdb where (finalpath like ? or finalpath like ?) order by timestamp DESC;"
					argv = (slashpath,path)
				else:
					select = "select * from xdelta3wrapperdb where finalpath=? order by timestamp DESC;"
					argv = (path)
				result = self.__DBW.resultQuery(select,argv);
				if (result != None):			
					versions = len(result)
					row = result[0]
					return versions, row[1], row[7]
				else:
					return None
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.', select, e)
			except ConnError, e:
				self.__log.critical('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()


	##@brief
	#
	#Search for a row with the path and the chtype = MOVE.
	#
	#@param path String with the path.
	#
	#@return True if there is a row in the xdelta3wrapperdb table.
	def wasMoved(self,path,timestamp=0):
		try:
			self.__DBW.openCon('/HdXDelta3WrapperDB')
			try:
				ts = " ";
				listArgs = [path]
				if (timestamp > 0):
					ts = " and timestamp <= ?"
					listArgs.append(str(timestamp))
				select = "select * from xdelta3wrapperdb where initpath like ? " + ts+" and chtype like 'MOVE' order by timestamp DESC;"
				result = self.__DBW.resultQuery(select,tuple(listArgs));
				if result:
					return True
				else:
					return False
			except IntegrityError, e:
				self.__log.critical('Integrity error has happened in the database.')
			except SQLError, e:
				self.__log.critical('SQL error has happened in the database: %s. Error: %s.', select, e)
			except ConnError, e:
				self.__log.warning('The connection to the database doesnt exits.')
		finally:
			self.__DBW.closeCon()


