#!/usr/bin/python
# -*- coding: UTF-8 -*-

#Copyright © 2008 Adrián Bravo

#This file is part of HDLorean.
#
#HDLorean is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#HDLorean is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with HDLorean.  If not, see <http://www.gnu.org/licenses/>.


import sys
import time
import os
import signal
import dbus, dbus.glib, dbus.service
import gobject
import traceback
from watcher import Watcher
from watcher import Scheduler
from watcher import BackendProxy
from watcher import CronHandler
from watcher import ConfigFileManager
from db import Journal


##
#
# @author Adrián, Danieloop
class hdloreand(dbus.service.Object,object):
	
	# Singleton pattern
	__instance = None
	__watcher = None
	__scheduler = None
	__cfm = None
	__bp = None
	__flagExit = False
	__dbusFile = None
	__exit = 0
	# GTK Main Loop
	__loop = None
	buss = dbus.SessionBus()
	print "Inicializado com.hdlorean.Server"
	name = dbus.service.BusName('com.hdlorean.Server', bus=buss)


	##
	#
	#
	def __init__(self):
		#HACK HACK HACK HACK
		dbus.service.Object.__init__(self, dbus.SessionBus(), '/hdlorean/signals')
		self.writePidFile()

	##
	#
	# @author Adrián
	def __new__(self):
		if self.__instance is None:
			self.__instance = object.__new__(self)
			# some inits
			# REMEMBER: Database inits must be first...
			db = Journal.Journal()
			db.createTable()
			self.__watcher = Watcher.Watcher()			
			self.__scheduler = Scheduler.Scheduler()
			bp = BackendProxy.BackendProxy(self.__watcher,self.__scheduler,self.__instance)
			self.__cfm = ConfigFileManager.ConfigFileManager()
		return self.__instance

	
	## @brief Write a file with the pid of the running process
	#
	# @author Adrián
	def writePidFile(self):
		pid = os.getpid()
		destPath = os.getenv("HOME")+"/.hdlorean/hdlorean.pid"
		# This is not necesary but its better this way
		if os.path.exists(destPath):
			os.remove(destPath)
			#TODO: If sum1 starts a second demon... crash!
		file = open(destPath,'w')
		file.write(str(pid))
		file.close()

	
	## @brief Delete the pid file 
	#
	# @author Adrián
	def deletePidFile(self):
		destPath = os.getenv("HOME")+"/.hdlorean/hdlorean.pid"
		os.remove(destPath)


	## @brief Makes some cleanup and exit the daemon properly
	#
	# @author Adrián
	def stopAndExit(self,exit_code='0'):
		# Watcher and Scheduler need some cleanup before exiting, and
		# we cannot trust on del method.
		self.__flagExit=True
		self.__exit = exit_code
		
	##
	#
	# @author Adrián
	def stopMon(self):
		self.deletePidFile()
		self.__watcher.close()
		self.__loop.quit()
		sys.exit(self.__exit)	


	@dbus.service.signal(dbus_interface='com.hdlorean.Signals',signature='')
	def signalStarted(self):
		pass
	
	@dbus.service.signal(dbus_interface='com.hdlorean.Signals',signature='')
	def signalStarted(self):
		pass

	@dbus.service.signal(dbus_interface='com.hdlorean.Signals',signature='s')
	def signalUnexpectedExit(self,trace):
		return trace

	## @brief Main function to execute when the mainloop
	# invokes it.
	# @author Adrián
	def run(self):
		try:
			if self.__flagExit:
				self.stopMon()			
			self.__scheduler.schedule()
			self.signalStarted()
			return True
		# Should we catch kill signal? We might finish cleaning
		# everything.
		except Exception, e:
			print "Unexpected exception. Exiting..."
			self.__exit=-1
			self.signalUnexpectedExit(str(traceback.format_exc()))
			self.stopMon()


	##
	#
	#
	def main(self):
		try:
			# Create a non-blocking loop to keep hdlorean running
			gobject.threads_init()
			self.__loop = gobject.MainLoop()
			# Adds the function to execute every 2 secs.
			gobject.timeout_add(3000,self.run)
			self.__loop.run()
		except KeyboardInterrupt,e:
			# Ctrl-C recieved and some cleanup must be done
			print "HD Lorean has received exit signal. Exiting..."
			self.__exit=0
			self.stopMon()


if __name__ == "__main__":
	daemon = hdloreand()
	daemon.main()
