# -*- coding: UTF-8 -*-

"""
Copyright (C) 2008 Salvador de la Puente

This file is part of HDLorean.

HDLorean is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

HDLorean is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with HDLorean.  If not, see <http://www.gnu.org/licenses/>.
"""

##@brief IStorageWrapper abstract class must be extended by classes that
# implement version management.
#
# @author Salvador de la Puente González
# @contact neo.salvador@gmail.com
class IStorageWrapper:
	"""
	IStorageWrapper abstract class must be extended by classes that
	implement version management.
	"""
	
	##@brief Constant name of the wrapper
	#
	__NAME = ""
	
	##@brief SnapshotManager instance.
	#
	__SM = None
	
	##@brief Default constructor takes the SnapshotManager object
	# that creates it.
	#
	#@param smObject SnapshotManager object that creates it.
	def __init__(self, smObject):
		pass
	
	##@brief Makes a backup for a set of files.
	#
	#@param fileList a list of tuples with 
	# id, timestamp, operation, init_path, final_path, requestor, wrote
	# sended from the Scheduler.
	#@param integral if it's setted true, force to make integral backups of
	# everything. By default it is false.
	def makeBackups(self, fileList, boolIntegral = False):
		"""
		Makes a backup for a set of files.
		"""
		pass
	
	##@brief Recover a file or a directory to a specific path.
	#
	#@param timestamp timestamp at the file or directory must be recovered.
	#@param originalPathList list of paths for the file(s) or directories
	# to recover.
	#@param recoverPath path where recovered file(s) are placed.
	#@return actual recover path where files(s) has been recovered.
	def recoverBackups(self, snapTimestamp, originalPathList, recoverPath):
		"""
		Recover a file or a directory to a specific path.
		"""
		pass
		
	##@brief Delete an interval of versions of a path between two timestamps.
	#
	#@param path path of the versions to delete
	#@param initTimestamp low timestamp of the interval. By default, it's 0.
	#@param finalTimestamp hight timestamp of the interval. By default, it's 0.
	def deleteBackups(self, path, initTimestamp = 0, finalTimestamp = 0):
		"""
		Delete an interval of versions of a path between two timestamps.
		"""
		pass