# -*- coding: UTF-8 -*-

#Copyright (C) 2008 Salvador de la Puente

#This file is part of HDLorean.
#
#HDLorean is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#HDLorean is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with HDLorean.  If not, see <http://www.gnu.org/licenses/>.


from timeLine import Snapshot
from snapshotManager import SnapshotManager
from db import History
##@brief Timeline is a borg class that lets frontend access to snapshot system
# 
# Timeline TODO: Completar comentarios
#
# @author Salvador de la Puente González
# @contact neo.salvador@gmail.com
class Timeline:
		
	##@brief First snapshots
	#
	__actualTimestamp = None
	
	##@brief SnapshotManager object
	#
	__db = None
	
	##@breif Set if iterator must be forward or backward
	#
	__backward = False
	
	##@brief Default constructor
	#
	def __init__(self):
		self.__backward = False
		self.__db = History.History()
		
	##@brief Set iteration to forward
	#
	# Note that you should call this method or its analogue, setBackward, before iterate.
	#
	def setForward(self):
		self.__backward = False
		
	##@brief Set iteration to backward
	#
	# Note that you should call this method or its analogue, setForward, before iterate.
	#
	def setBackward(self):
		self.__backward = True
		
	##@brief Iterator constructor
	#
	def __iter__(self):
		if self.__backward:
			actualTimestamp = self.__db.lastTimestamp()
		else:
			actualTimestamp = self.__db.firstTimestamp()
			
		return self
	
	##@brief Make a new snapshot
	#
	#@param pathList list of paths to make the snapshot
	#@param integral boolean to force integral or not
	def makeSnapshot(self,pathList,integral):
		pass #TODO: Deal with scheduler		
	
	##@brief Next implementation for iteration
	#
	#@raise StopIteration to control iteration process
	def next(self):
		if self.__backward:
			actualTimestamp = self.__db.prevTimestamp(actualTimestamp)
		else:
			actualTimestamp = self.__db.nextTimestamp(actualTimestamp)
			
		if not actualTimestamp:
			raise StopIteration
		
	##@brief If index is a slice, get item return a list of snapshots between timestamps given.
	#
	#@param key timestamp of the snapshot to recover or a slice object where its bounds are timestamps in some range
	#@return if key is a timestampt, the method returns a valid snapshot or None, if does not exist a Snapshot with specified timestamp.
	# If key is a slice, the method return the most aproximate range between bounds. 
	#@raise IndexError to control loops and invalid index
	#@raise TypeError if key is neither slice object nor int
	def __getItem__(self, key):
		# Slice key
		if isinstance(key, slice):
			# Search for range inside the History
			result = self.__db.giveBetween(key.start,key.stop)
			
			# If no range, IndexError
			if not result:
				raise IndexError
			
			# Return adjusted range of snapshots
			resultDictionary = {}
			for entry in result:
				newItem[entry[0]] = Snapshot.Snapshot(entry[0])
		
		# Integer key	
		elif isinstance(key, int):
			# Search for a key
			result = self.__db.giveBetween(key,key)
			
			# If it does not exist, IndexError
			if not result:
				raise IndexError
		
			return Snapshot.Snapshot(result[0][0])
	
		# Else, typeError
		else:
			raise TypeError
				