# -*- coding: UTF-8 -*-

#Copyright (C) 2008 Diana Lopez, Daniel Tabas, Salvador de la Puente.

#This file is part of HDLorean.
#
#HDLorean is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#HDLorean is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with HDLorean.  If not, see <http://www.gnu.org/licenses/>.


import Watcher
import ConfigFileManager as CFM
import Scheduler
import dbus, dbus.glib, dbus.service
from snapshotManager import SnapshotManager

class BackendProxy(dbus.service.Object):
	## We use this variable to initialize dbus.service.Object once
	__dbusStarted = False
	
	__conf = None
	__wat = None
	__sche = None
	__snap = None
	def __init__(self,wat,sch,daem):
		if not self.__dbusStarted:
			self.__dbusStarted = True		
			dbus.service.Object.__init__(self, dbus.SessionBus(), '/hdlorean')
			print "Inicializado dbus"
			self.__conf = CFM.ConfigFileManager()
			self.__wat = wat
			self.__sche = sch
			self.__snap = SnapshotManager.SnapshotManager()
			self.__daemon = daem
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='as')
	def getWatchedFiles(self):
		return self.__conf.getWatchedFiles()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='ssss',out_signature='')
	def setFileLists(self, whitelist, reWhitelist, blacklist, reBlacklist):
		return self.__conf.setFileLists(whitelist.encode("utf-8","strict"), reWhitelist.encode("utf-8","strict"), blacklist.encode("utf-8","strict"), reBlacklist.encode("utf-8","strict"))
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='as')
	def getPReWhiteList(self):
		return self.__conf.getPReWhiteList()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='as')
	def getPReBlackList(self):
		return self.__conf.getPReBlackList()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='as')
	def getPWhiteList(self):
		return self.__conf.getPWhiteList()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='as')
	def getPBlackList(self):
		return self.__conf.getPBlackList()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='as')
	def getParentsList(self):
		return self.__conf.getParentsList()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='as')
	def getOptions(self,section):
		return self.__conf.getOptions(section)
		
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='as')
	def getSections(self):
		return self.__conf.getSections()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='ss', out_signature='b')
	def hasOption(self,section,option):
		return self.__conf.hasOption(section,option)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='b')
	def hasSection(self,section):
		return self.__conf.hasSection(section)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='ss', out_signature='')
	def removeConfigItem(self,section,option):
		return self.__conf.removeConfigItem(section,option)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='b')
	def removeSection(self,section):
		return self.__conf.removeSection(section)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def addSection(self,section):
		return self.__conf.addSection(section)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='s')
	def getLanguage(self):
		return self.__conf.getLanguage()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def setLanguage(self,lang):
		self.__conf.setLanguage(lang)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='b')
	def getNautilusIntegration(self):
		return self.__conf.getNautilusIntegration()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def setNautilusIntegration(self,nautint):
		self.__conf.setNautilusIntegration(nautint)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='s')
	def getPeriodically(self):
		return self.__conf.getPeriodically()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def setPeriodically(self,period):
		self.__conf.setPeriodically(period)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='b')
	def getEveryChange(self):
		return self.__conf.getEveryChange()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def setEveryChange(self,every):
		self.__conf.setEveryChange(every)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='x')
	def getDeleteIfSpace(self):
		return self.__conf.getDeleteIfSpace()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def setDeleteIfSpace(self,delifspace):
		self.__conf.setDeleteIfSpace(delifspace)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='x')
	def getDeleteIfOlder(self):
		return self.__conf.getDeleteIfOlder()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def setDeleteIfOlder(self,delifolder):
		self.__conf.setDeleteIfOlder(delifolder)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='s')
	def getBackupDirectory(self):
		print self.__conf.getBackupDirectory()
		return self.__conf.getBackupDirectory()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')	 
	def setBackupDirectory(self,backupdir):
		print 'CALLED'+backupdir
		self.__conf.setBackupDirectory(backupdir.encode("utf-8","strict"))
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='s')
	def getStartDir(self):
		return self.__conf.getStartDir()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')	 
	def setStartDir(self,startdir):
		self.__conf.setStartDir(startdir.encode("utf-8","strict"))
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='s')
	def getBlackList(self):
		return self.__conf.getBlackList()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def setBlackList(self,blacklist):
		self.__conf.setBlackList(blacklist)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='s')
	def getWhiteList(self):
		return self.__conf.getWhiteList()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def setWhiteList(self,whitelist):
		self.__conf.setWhiteList(whitelist.encode("utf-8","strict"))
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='s')
	def getReWhiteList(self):
		return self.__conf.getReWhiteList()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def setReWhiteList(self,rewhitelist):
		self.__conf.setReWhiteList(rewhitelist)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='s')
	def getReBlackList(self):
		return self.__conf.getReBlackList()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def setReBlackList(self,reblacklist):
		self.__conf.setReBlackList(reblacklist)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='i')
	def getBatteryChargeLimit(self):
		return self.__conf.getBatteryChargeLimit()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def setBatteryChargeLimit(self,battchargelimit):
		self.__conf.setBatteryChargeLimit(battchargelimit)
	
		
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='x')
	def getCpuUseLimit(self):
		return self.__conf.getCpuUseLimit()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def setCpuUseLimit(self,cpuuselim):
		self.__conf.setCpuUseLimit(cpuuselim)
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='i')
	def getMaxBackupSize(self):
		return self.__conf.getMaxBackupSize()
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='s', out_signature='')
	def setMaxBackupSize(self,maxbackupsize):
		self.__conf.setMaxBackupSize(maxbackupsize)
	
#COMENTADO HASTA QUE SEPAMOS COMO FUNCIONARÁ CRON	
	#@dbus.service.method(dbus_interface='com.hdlorean.Watcher')
	#def cronCommand(self):
		#return self.__wat.cronCommand()


	@dbus.service.method(dbus_interface='com.hdlorean.Scheduler',in_signature='', out_signature='av')
	def getInfo(self):
		return self.__sche.getInfo()
		
	
	##@brief Recover a file or a directory to a specific path
	#
	#@param snapTimestamp max timestamp for the file or directory to recover
	#@param originalPathList path of the file(s) (all with the same parent directory) or directory to recover
	#@param recoverPath place where recovered file(s) are placed
	#@return actual recover path where files(s) are recovered
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot',in_signature='xass', out_signature='s')
	def recoverBackups(self, snapTimestamp, originalPathList, recoverPath):
		return self.__snap.recoverBackups(snapTimestamp, originalPathList, recoverPath)
	
	
	##@brief Get a list of contents of a directory along all of the time line
	#
	#@param path path of the directory to list
	#@return a list of tuples with paths inside the directory and type of the path ('F' for a file or 'D' for a directory)
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot',in_signature='s', out_signature='a(sssx)')
	def getCompleteDirList(self, path):
		return self.__snap.getDeepDirList(path)
	
	##@brief Get info about a file at some time
	#
	# Note that file can be a directory but then, it will be treated like a file
	#
	#@param snapTimestamp timestamp of the snapshot 
	#@param path path of file or directory 
	#@return a tuple with file name (string), file path (string), monitorized indicator (boolean), file permissions (octal integer), file size (integer), last operation date (string), last operation (string), backup performed (integral, differential, directory, not necesary), file type (F for files or D for directories), number of backups, and UNIX timestamp
	# For each tuple, if file or directory was not monitorized then, info after monitorized indicator is 0 or '' and monitorized indicator is False.
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot',in_signature='xs', out_signature='ssbxxssssxx')
	def getFileInfo(self, snapTimestamp, path):
		return self.__snap.getFileInfo(int(snapTimestamp),path)
	
	##@brief Gets the historical of versions for a path.
	#
	# The historical of a path is a list with all versions tracked
	# over the path and same info of getVersionInfo.
	#
	#@param path path of file or directory. If path is a directory then,
	# operations over sons of that directory are also shown inside historical.
	#
	#@return a pair with a list of tuples with the next information
	# ID
	# timestamp
	# init path
	# operation path
	# location
	# operation
	# requestor
	# backup size
	# name of sm
	# file size
	# permissions
	# x permissions
	# mime type
	# last access
	# last modif
	# creation time
	# type (F or D)
	#
	# And total number of backups too.
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot', 
						 in_signature='s',
						 out_signature='a(xxsssssxsxxxsxxxs)x')
	def getHistorical(self, path): 
		"""
		Gets the historical of versions for a path.
		"""
		return self.__snap.getVersionsInfo(path.encode("utf-8","strict"))
	
	##@brief Gets a quick briefing of all versions of a path
	#
	# The briefing some quick info about the versions of a path
	#
	#@param path path of file or directory. If path is a directory then,
	# operations over sons of that directory are also shown inside briefing.
	#
	#@return a pair with a list of tuples with the next information
	# timestamp
	# operation path
	# operation
	# backup size
	# file size
	# amount of backed up data*
	#
	# And total number of backups too.
	#
	# If version has more than 1000 files, -1 is returned
	# TODO: Extends note with average of data size and composition	
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot', 
						 in_signature='s',
						 out_signature='a(xssxxx)x')
	def getVersionsBriefing(self, path):
		"""
		Gets a quick briefing of all versions of a path
		"""
		return self.__snap.getVersionsBriefing(path.encode("utf-8","strict"))
	
	##@brief Gets amount of data backed up in a version
	#
	# The briefing some quick info about the versions of a path
	#
	#@param path path of the version
	#@param timestamp time of the versio
	#
	#@return amount (in bytes) of data backed up in a version
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot', 
						 in_signature='sx',
						 out_signature='x')
	def getAllDataSizeInVersion(self, path, timestamp):
		"""
		Gets amount of data backed up in a version
		"""
		return self.__snap.getAllDataSizeInVersion(path, timestamp)
	
	##@brief Gets GMT and DST for a given timestamp
	#
	#@param timestamp reference timestamp
	#@return a tuple with GMT and DST for a nearest timestamp to the given
	# timestamp. If GMT and DST are both -1, timestamp has no attributes.
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot', 
						 in_signature='x',
						 out_signature='xx')
	def getTimeAttributes(self, timestamp):
		"""
		Gets GMT and DST for a given timestamp
		"""
		return self.__snap.getTimeAttributes(timestamp)
	
	##@brief Gets version info for a path and timestamp.
	#
	#@param path path of file or directory. If path is a directory then,
	# operations over sons of that directory are also shown inside historical.
	#
	#@param timestamp reference timestamp of the version. 
	#
	#@return a pair with a list of one tuple with the next information
	# ID
	# timestamp
	# init path
	# operation path
	# location
	# operation
	# requestor
	# backup size
	# name of sm
	# file size
	# permissions
	# x permissions
	# mime type
	# last access
	# last modif
	# creation time
	# type (F or D)
	#
	# And total number of backups too.
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot', 
						 in_signature='sx',
						 out_signature='a(a(xxsssssxsxxxsxxxs)x)')
	def getVersionInfo(self, path, timestamp): 
		"""
		Gets version info for a path and timestamp.
		"""
		return self.__snap.getVersionsInfo(path.encode("utf-8","strict"), timestamp)
			
	##@brief It returns the most recent snapshot timestamp
	#
	#@return the most recent snapshot timestamp or 0 there are no snapshots performed			
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot',in_signature='', out_signature='x')
	def lastTimestamp(self):
		return self.__snap.lastTimestamp()
	
	##@brief It returns the very first snapshot timestamp
	#
	#@return the very first snapshot timestamp or 0 there are no snapshots performed
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot',in_signature='', out_signature='x')
	def firstTimestamp(self):
		return self.__snap.firstTimestamp()
	
	##@brief It returns the inmediatly previous snapshot timestamp of another
	#
	#@param snapTimestamp reference timestamp
	#@return the inmediatly previous snapshot timestamp of another or None if it does not exist	
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot',in_signature='x', out_signature='x')
	def prevTimestamp(self, snapTimestamp):
		return self.__snap.prevTimestamp(snapTimestamp)
	
	##@brief It returns the inmediatly next snapshot timestamp of another
	#
	#@param snapTimestamp reference timestamp 
	#@return the inmediatly next snapshot timestamp of another or None if it does not exist
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot',in_signature='x', out_signature='x')
	def nextTimestamp(self, snapTimestamp):
		return self.__snap.nextTimestamp(snapTimestamp)
	
	##@brief It returns a list of timestamps between two bounds
	#
	#@param startSnapTimestamp low snapshot bound. It can be None to reference the first timestamp
	#@param stopSnapTimestamp hight snapshot bound. It can be None to reference the last timestamp
	#@return a list of timestamps between two bounds
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot',in_signature='xx', out_signature='as')
	def getTimestampsBetween(self,startSnapTimestamp,endSnapTimestamp):
		return self.__snap.getTimestampsBetween(startSnapTimestamp,endSnapTimestamp)

	@dbus.service.method(dbus_interface='com.hdlorean.Interface',in_signature='', out_signature='')
	def stopDaemon(self):
		return self.__daemon.stopAndExit()
	
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot',in_signature='xsbb', out_signature='x')
	def getLastBackup(self,timestamp,path,integral,recursive):
		return self.__snap.getLastBackup(timestamp,path,integral,recursive)

	@dbus.service.method(dbus_interface='com.hdlorean.Watcher',in_signature='s', out_signature='')
	def snapshotRequest(self,user):
		return self.__wat.snapshotRequest(user.encode("utf-8","strict"))
	
	
	@dbus.service.method(dbus_interface='com.hdlorean.Snapshot',in_signature='sxx', out_signature='')
	def deleteBackups(self,path,init_timestamp,final_timestamp):
		return self.__snap.deleteBackups(path,init_timestamp,final_timestamp)
