# -*- coding: UTF-8 -*-

#Copyright (C) 2008 Adrián Bravo

#This file is part of HDLorean.
#
#HDLorean is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#HDLorean is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with HDLorean.  If not, see <http://www.gnu.org/licenses/>.


from watcher import BatteryMonitor
from snapshotManager import SnapshotManager
import time
from db import Journal
import gobject
from hdlogger import HDLogger
import dbus, dbus.glib, dbus.service
import thread


## @brief Schedules the actions of hdlorean based on info about the
# system status and the remaining tasks.
#
# @author Adrián
class Scheduler(dbus.service.Object):

	
	__battMonitor = None
	__snapshotManager = None
	## Maximun number of rows we can keep on journal without lacking
	# information about changes on the syste information about changes on the systemm
	__maxJournalRows = 300
	## Lowest battery level. If we reach this value
	# The scheduler delays the tasks.
	__minBattLevel = 20
	__journal = None
	##
	__logger = None
	__log = None
	## Lock to force wait until sending new snapshot request
	__msLock = None
	# ids of the snapshots being made
	__inProgress = None
	# min number of rows to plan
	__minRows = 15


	## @brief Initializes the journal and the monitors
	#
	# @author Adrián
	def __init__(self):
		dbus.service.Object.__init__(self, dbus.SessionBus(), '/hdlorean/signalsSch')
		self.__logger = HDLogger.HDLogger()
		self.__inProgress = set()
		# create lock for sync
		self.__msLock = thread.allocate_lock()
		self.__log = self.__logger.setupHDLogger('hdlorean.Watcher.Scheduler')
		self.__battMonitor = BatteryMonitor.BatteryMonitor()
		self.__journal = Journal.Journal()
		self.__snapshotManager = SnapshotManager.SnapshotManager()
		#Configure snapshot manager to work with xdelta3
		self.__snapshotManager.setup("xdelta3")

	
	## @brief Schedule operations of the hdlorean daemon.
	#
	# @author Adrián
	def schedule(self):
		# If we've exceed the maxJournalRows limit we must
		# start the work, although the Monitors stay against.
		print "Sched"
		result_set = self.__journal.getOldest()
		if (result_set is not None) and (result_set != []) and (result_set != ['']):
			result_set = set(result_set)
			result_set.difference_update(self.__inProgress)
			length = len(result_set)
			if length > self.__maxJournalRows:
				#self.__log.info("Maximun number of rows exceeded, starting the job.")
				# Empty done jobs
				#self.__journal.freeJournal()
				# Start with the oldest task
				self.__inProgress.update(result_set)
				thread.start_new_thread(self.__makeSnapshot,(result_set,))
			elif length > 0:
				# Asks monitors
				# TODO: Ask CPU and Disk Monitors
				if (self.__battMonitor.getAcStatus() != "on") and \
						(self.__battMonitor.getRemainingCharge() <= self.__minBattLevel):
					pass
				else:
					self.__inProgress.update(result_set)
					thread.start_new_thread(self.__makeSnapshot,(result_set,))


	## @brief Insert the witness in the Journal and orders a new snapshot.
	#
	# Insert a witness in the Journal and commands a new snapshot.
	# @author Adrián
	def __makeSnapshot(self,tids):
		# Lets work
		# Insert the witness
		self.__journal.insertSpecialRow("0")
		self.__msLock.acquire()
		# Call Snapshot method and wait for confirmation
		if len(tids):
			self.__snapshotManager.makeBackups(list(tids),False)
		# When ack recieved:
		# mark the witness as wrote
		self.__journal.setWroteSR()
		self.__inProgress.difference_update(tids)
		self.__msLock.release()
		self.makeSnapshot()


	## @brief Signal to refresh the gui.
	#
	# @author Daniel Tabas
	@dbus.service.signal(dbus_interface='com.hdlorean.SignalsSch')
	def makeSnapshot(self):
		pass


	##@brief marks snapshots as wrote
	#
	# @param idlist: List of integers that identify the snapshots being marked
	# @author Adrián
	def setWrote(self,idlist):
		self.__journal.setWrote(idlist)
