# -*- coding: UTF-8 -*-

#Copyright (C) 2008 Adrián Bravo, Jorge Quintás, Diana Lopez

#This file is part of HDLorean.
#
#HDLorean is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#HDLorean is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with HDLorean.  If not, see <http://www.gnu.org/licenses/>.


from pyinotify import EventsCodes,ProcessEvent,ThreadedNotifier,WatchManager
import ConfigFileManager as CFM
from PyinotifyHandler import PyinotifyHandler
from db import Journal
from snapshotManager import SnapshotManager
import time
import CronHandler as CH
from hdlogger import HDLogger
import os
import GlobalExceptions
import fnmatch
import thread


## @brief provides functions to treat events notification 
#
# Watcher inherits from ProcessEvent and overrides the process
# of the super class to bring the desired functionality to this module
# @author: Jorge, Adrián
class Watcher(ProcessEvent):


	## set of files/dirs that we don't want to monitorize
	__pblacklist = None
	__blacklist = None
	## set of files/dirs that we want to monitorize
	__whitelist = None
	## set of blacklisted patterns
	__reBlacklist = None
	## instance of pyinotifyHandler to interact with
	__pyinotifyHandler = None
	__cronHandler = None
	## database handler
	__journal = None
	##things for moves...
	__moveHelper = ''
	##
	__logger = None
	##
	__log = None
	##
	__CFM = None
	#mew
	__watched = None
	__snapshotManager = None
	#ugly thing
	__lastPath = ''
	#flag indicating the status of the notifier running/stopped
	__isRunning = False
	#__srLock = None
	#__bniLock = None
	#pattern list
	__patterns = None

	## @brief Initializes the CronHandler, the PyinotifyHandler and the Watched Files.
	#
	# 
	# @author: Jorge, Adrián
	def __init__(self):
		self.__logger = HDLogger.HDLogger()
		self.__log = self.__logger.setupHDLogger('hdlorean.Watcher.Watcher')
		# Create new locks
		#self.__srLock = thread.allocate_lock()
		#self.__bniLock = thread.allocate_lock()
		self.__snapshotManager = SnapshotManager.SnapshotManager()
		self.__snapshotManager.setup("xdelta3")
		self.__CFM = CFM.ConfigFileManager(self)
		self.__CFM.findFiles()
		self.__pblacklist = self.__CFM.getPBlackList()
		self.__blacklist = self.__CFM.getBlackList()
		self.__whitelist = self.__CFM.getWhiteList()
		self.__reBlacklist = self.__CFM.getPReBlackList()
		parents = self.__CFM.getParentsList()
		self.__watched = self.__CFM.getWatchedFiles()
		self.__patterns = self.__CFM.getReBlackList()
		self.__journal = Journal.Journal()
		# Make the backups of the new files added while the daemon wasnt running.
		self.backupNewItems(self.__watched)
		self.__pyinotifyHandler = PyinotifyHandler(self,self.__watched,self.__pblacklist,self.__whitelist,self.__reBlacklist,parents,self.__CFM.getEveryChange())
		self.__log.info('Succesfully initialized')
		if(self.__CFM.getEveryChange()):
			try:
				self.__pyinotifyHandler.startNotifier()
			except GlobalExceptions.NotifierRunning:
				self.__log.warning('Notifier already running')
			self.__log.info('Started')
			self.__isRunning = True
		#Creates the file with the env vars for the cron
		self.__cronHandler = CH.CronHandler(self)
		# Insert the task in the crontab if needed
		self.updateCrontab()
		


			
	## @brief Before leaving, watcher needs to make some cleanup on pynotify.
	#
	# Some cleanup before destroying Watcher
	# @author Adrián
	def close(self):
		self.__log.info('Stop monitoring')
		try:
			self.__pyinotifyHandler.stopNotifier()
		except GlobalExceptions.NotifierStopped:
			self.__log.warning('Attemp to stop notifier, but notifier was already stopped')


	## @brief fills the queue with events attemping to whitelist and blacklist info
	#
	# If an event is raised on a blacklisted file or directory, the event is not
	# queued
	# @author: Jorge
	def process_default(self,event,chtype='',init='',final=''):
		#we do nothing with IN_IGNORED events
		#FIXME: IN_MOVED_FROM...this is a hack...
		self.__lastPath = event.path+'/'+event.name
		if event.event_name == 'IN_IGNORED'or event.event_name =='IN_MOVED_FROM'or event.event_name == 'IN_MOVED_TO':
			pass
		else:
			#first of all, we have to get the real path of the object
			#for SELF events, the event.name is empty, so we don't append it to the
			#event path
			if event.event_name == 'IN_DELETE_SELF':
				str = event.path
			else:
				#non SELF events store the name of the object that rises the event in
				#event.name, we  need to reconstruct the full path this way
				str = event.path+'/'+event.name
			#here begins the treatment, discard the blacklisted files and directories
			if str in self.__pblacklist or str in self.__reBlacklist:
				pass
			else: #FIXME:Fede dice "Jorge, guarrete, se hace con int(time.time())" 
				timestamp = int(time.time())
				self.__journal.insert(timestamp,chtype,init,final,'inotify','0')


	## @brief processes deleted events
	#
	# Informs of a delete event on a directly watched file or dir and calls the default process
	# @see: process_default
	# @param event obect that contains the relevant information about the raised event
	# @author Jorge
	def process_IN_DELETE_SELF(self,event):
		filePath = event.path+'/'+event.name
		#root,ext = os.path.splitext(filePath)
		#if '.' not in root:
		if self.__checkLists(filePath):
			pass
		else:
			if(event.is_dir):
				self.__log.info('*dir deleted ==> '+ event.path)
				print '*dir deleted ==> '+ event.path
			else:
				#FIXME:we're not directly watching files. If we start doing that, we must 
				#try the next instruction, I'm not sure of this way to obtain the path
				self.__log.info('*file deleted ==> '+event.path + '/' + event.name)	
				print '*file deleted ==> '+event.path + '/' + event.name
			self.process_default(event,'DELETE',event.path + '/' + event.name,event.path + '/' + event.name)
		
			
	## @brief processes modify events
	#
	# Informs of a modify event and calls the default process
	# @see: process_default
	# @param event object that contains the relevant information about the raised event			
	# @author Jorge
	def process_IN_MODIFY(self,event):
		if event.path+'/'+event.name != self.__lastPath:
			path = ''
			if event.is_dir:
				path = event.path
			else:
				path = event.path+'/'+event.name
			#filePath = event.path+'/'+event.name
			#bwajaja, split the path into root and extension
			#dirPath = event.path
			#black = self.__blacklist
			#reBlack = self.__reBlacklist
			#root,ext = os.path.splitext(filePath)
			#if '.' not in root:
			if self.__checkLists(path):
				pass
			else:
				if(event.is_dir):
					#FIXME: check if this is possible, I never seen a modify event raised against a dir
					#if this happen process_default must take care of not process the event twice
					self.__getContent(path, 'MODIFY', path, path, 'inotify')
					self.__log.info('dir modified ==> ' + event.path + '/' + event.name)
					print 'dir modified ==> ' + event.path + '/' + event.name
				else:
					self.__log.info('file modified ==> '+ event.path + '/' + event.name)
					print 'file modified ==> '+ event.path + '/' + event.name
				self.process_default(event,'MODIFY',event.path + '/' + event.name,event.path + '/' + event.name)
		
		   
	## @brief processes creation events
	#
	# Informs of a creation event and calls the default process
	# @see: process_default
	# @param event object that contains the relevant information about the raised event
	# @author Jorge		   
	def process_IN_CREATE(self,event):
		path = event.path+'/'+event.name
		#if event.is_dir:
		#	path = event.path
		#else:
		#	path = event.path+'/'+event.name
		#root,ext = os.path.splitext(filePath)
		#if '.' not in root:
		if self.__checkLists(path):# or self.__checkLists(dirPath):
			pass
		else:
			#FIXME: print before testing if is right, move this
			if event.is_dir:
				self.__log.info('dir created ==> ' +event.path + '/' + event.name) 
				print 'dir created ==> ' +event.path + '/' + event.name
				#self.__pyinotiftHandler.addToWatched(event.path + '/' + event.name)
			else:
				self.__log.info('file created ==> '+event.path + '/' + event.name)
				print 'file created ==> '+event.path + '/' + event.name
			self.process_default(event,'CREATE',event.path + '/' + event.name,event.path + '/' + event.name)
			

	## @brief processes deleted events
	#
	# Informs of a delete event into a watched directory and calls the default process. Directories are
	# directly watched, so a delete event raises two inotify events, IN_DELETE and IN_DELETE_SELF. To avoid
	# this behavior( or behaviour xD) we ignore IN_DELETE for directories
	#FIXME: we can do this in PyinotifyHandler by adding only the relevant flags to files and directories separately
	# @see process_default
	# @param event object that contains the relevant information about the raised event	
	# @author Jorge		
	def process_IN_DELETE(self,event):
		path = ''
		if event.is_dir:
			path = event.path
		else:
			path = event.path+'/'+event.name
		#root,ext = os.path.splitext(filePath)
		#if '.' not in root:		
		if self.__checkLists(path):
			pass
		else:
			if not event.is_dir:
				self.__log.info('file deleted ==> ' + event.path + '/' + event.name)
				print 'file deleted ==> ' + event.path + '/' + event.name
			self.process_default(event,'DELETE',event.path + '/' + event.name,event.path + '/' + event.name)


	##@brief private method to obtain the content of a directory
	#
	# @author Jorge
	# deprecated
	def __recursiveGetContent(self,current):
		result = []
		if os.path.isdir(current):
			#add to result list the current content
			try:
				relativeCurrent = os.listdir(current)
				for item in relativeCurrent:
					item = os.path.join(current,item)
					if not self.__checkLists(item):
					   result = result + self.__recursiveGetContent(item)
			except OSError, e:
				#Some kind of permission fail
				self.__log.info('Cant access '+current+ '. It will never be backed up')	
		else:
			result = result + [current]
		return result
	
	##@brief Explore the dirtree finding files and dirs
	#
	# First, we obtain the content of the current directory, and for each directory, we use
	# __recursiveGetContent to obtain all the subtree from the initial path
	# @param path the start point. It must be the path of a directory
	# @return CompleteContent contains all files and directories under path
	# @author Jorge
	#FIXME: check if path is valid and add error treatment. Some fails detected with 
	#dirs that have a short lifetime. bzr pull or bzr push on watched dirs breaks
	def __getContent(self,path,event,init,final,requestor):
		timestamp = int(time.time())
		data = {}
		data['timestamp'] = timestamp
		data['event'] = event
		data['init'] = init
		data['final'] = final
		data['requestor'] = requestor		
		os.path.walk(path, self.__visitor,data)		
	

	##@brief processes moves into directory
	#
	# Is executed when a file or directory is moved to a watched dir. This method is also used to
	# watch rename events
	# @param event object that contains the relevant information about the raised event
	# @autor Jorge
	def process_IN_MOVED_TO(self,event):
		## distinguish between moves and renames!!or maybe not
		#if the event is raised from a dir get all the content of the dir to include it
		#in the journal for snapshot-manager magic
		#filePath = event.path+'/'+event.name
		#dirPath = event.path
		#black = self.__blacklist
		#reBlack = self.__reBlacklist
		#root,ext = os.path.splitext(filePath)
		#if '.' not in root:
		path = ''
		#if event.is_dir:
		#	path = event.path
		#else:
		#	path = event.path+'/'+event.name
		path = event.path+'/'+event.name
		if self.__checkLists(path):
			pass
		else:
			completeContent = []
			if(event.is_dir):
				#get the content of the current dir
				#FIXME: this fails when the dir has a short life. test it running hdlorean and making bzr pull into a watched
				#location
				self.__getContent(path,'MOVE',self.__moveHelper,event.path + '/' + event.name,'inotify')
			#now, take all the info into the journal
			#begin with the object that caused the notification
			self.__log.info('moved into watched directory'+ event.path +'/'+ event.name)	
			print 'moved into watched directory'+ event.path +'/'+ event.name
			#self.process_default(event,'MOVE',self.__moveHelper,event.path + '/' + event.name)
			#if it was a dir, and contains something, move the info to the journal
			#for path in completeContent:
			#	self.process_default(event,'MOVE','',path)
			# before the magic, __moveHelper is set to '' in order to interfer as less as possible
			# with movements that are not renames between watched directories 	
			self.__moveHelper=''


	## @brief A new snapshot must be made
	#
	# Inserts in journal all watched files to make a snapshot
	# @param sender The requestor of the snapshot. Can be USER or CRON.
	# @author: Adrián
	def snapshotRequest(self,sender):
		thread.start_new_thread(self.__threadedSnapshotRequest,(sender,))
	

	## @brief Inserts in journal all the neccesary to acomplish a new snapshot
	#
	# TODO: FIX IT, block db, guarantee atomicity
	# @param sender The requestor of the snapshot. Can be USER or CRON
	# @author Adrián, Diana
	def __threadedSnapshotRequest(self,sender):
		#self.__srLock.acquire()
		
		for path in self.__watched:
			self.__getContent(path, sender, path, path, sender)
			#self.__journal.insert(timestamp,sender,path,path,'inotify','0')
			#for item in completeContent:
			#	self.__journal.insert(timestamp,sender,item,item,'inotify','0')
		#self.__srLock.release()

	
	##@brief process changes in attributes
	#
	# @author Jorge
	def process_IN_ATTRIB(self,event):
		if event.path+'/'+event.name != self.__lastPath:
			path = ''
			if event.is_dir:
				path = event.path
			else:
				path = event.path+'/'+event.name
			if self.__checkLists(path):
				pass
			else:
				self.__log.info('atribute changed '+path)
				print 'atribute changed '+ path
	

	##@brief for movements that begins in watched directories
	#
	# @autor Jorge
	def process_IN_MOVED_FROM(self,event):
		path = ''
		#if event.is_dir:
		#	path = event.path
		#else:
		#	path = event.path+'/'+event.name
		path = event.path+'/'+event.name
		if self.__checkLists(path):
			pass
		else:
			self.__log.info('moved out of a watched directory '+event.path+'/'+event.name)
			self.__moveHelper = event.path+"/"+event.name
			print 'moved out of a watched directory '+event.path+'/'+event.name
	

	## @brief Update crontab perodicity.
	#
	# @author Adrián
	def updateCrontab(self):
		self.__cronHandler.addTask()


	##@brief marks snapshots as wrote
	#
	# @param idlist: List of integers that identify the snapshots being marked
	# @author Jorge
	def setWrote(self,idlist):
		try:
			self.__journal.setWrote(idlist)
		except IntegrityError:
			self.__log.info('row already marked with setWrote')
		except SQLError:
			pass
		except ConnError:
			pass

		
	##@brief callback function witch reload the necessary lists
	#
	#@author Jorge
	def updateLists(self):
		#first of all, obtain the old sets to extract the differences with the new ones
		oldParents = self.__pyinotifyHandler.getParentList()
		oldWatched = self.__watched
		#store locally the new ones from ConfigFileManager
		newWatched = self.__CFM.getWatchedFiles()
		newParents = self.__CFM.getParentsList()
		#calculate the items to be added/removed
		toAddInWatched = newWatched.difference(oldWatched)
		#obtain the elements that are no longer in watchedlist
		toRemoveFromWatched = oldWatched.difference(newWatched)
		# same as above with parent list
		toAddInParents = newParents.difference(oldParents)
		toRemoveFromParents = oldParents.difference(newParents)
		#update local lists
		self.__pblacklist = self.__CFM.getPBlackList()
		self.__whitelist = self.__CFM.getWhiteList()
		self.__reBlacklist = self.__CFM.getPReBlackList()
		self.__watched = newWatched
		self.__patterns = self.__CFM.getReBlackList()
		self.__log.debug('to add in watched '+ repr(toAddInWatched))
		self.__log.debug('to remove from watched' + repr(toRemoveFromWatched))
		print 'to remove from watched' + repr(toRemoveFromWatched)
		#self.__log.info()
		self.__pyinotifyHandler.updateWatched(self.__pblacklist,self.__whitelist,self.__reBlacklist,toAddInWatched,toRemoveFromWatched,toAddInParents,toRemoveFromParents)
		#start and stop notifier
		if(self.__CFM.getEveryChange()):
			if not self.__isRunning:
				self.__pyinotifyHandler.start()
				#self.__pyinotifyHandler.update(True)
				self.__isRunning = True
		else:
			if self.__isRunning:
				self.__pyinotifyHandler.stop()
				#self.__pyinotifyHandler.update(False)
				self.__isRunning = False
		# Make the first snapshot of all the items that have been added
		self.backupNewItems(toAddInWatched)


	##@brief Check if a file or directory is in a blacklist or reBlacklist
	#
	#@param path of the file or directory that we want to check
	#@return true if path is blacklisted, false in other case	
	def __checkLists(self,path):
		blacklisted = False
		#backup directory is blaclisted
		if path.startswith(self.__CFM.getBackupDirectory()+'/'):
			return True
		#check if path matches with an excluded regexp
		#for item in self.__patterns:
		#TODO what about the promised split!!!!!xD. Do ii
		patterns = self.__patterns.split(';')
		for item in patterns:
			if fnmatch.fnmatch(path,item):
				return True
		#check if path is on the blacklist
		if path in self.__pblacklist:
			return True
		else:
			#check if path is contained on the blacklist
			for item in self.__pblacklist:
				if item != '' and  path.startswith(item+'/'):
					return True
		#./hdlorean cotains our databases, is a good idea to blacklist the dir
		if path.startswith(os.getenv("HOME")+"/.hdlorean"): 
			return True
		#chech if the path is on the pattern blacklist
		for item in self.__reBlacklist:
			#check if an item of the pattern blacklist is a substring of the path of the event
			if item != '' and path.startswith(item+'/'):
				return True			
		return blacklisted


	## @brief Checks if there is any backup of the items received and, if necesary
	# insert the path in the journal so its first backup become acomplished.
	#
	# This function just calls the real one in a thread.
	# @see __threadedBackupNewItems
	# @param items The new paths that have been added to the watched ones
	# @author Adrián
	def backupNewItems(self,items):
		thread.start_new_thread(self.__threadedBackupNewItems,(items,))


	## @brief Checks if there is any backup of the items received and, if necesary
	# insert the path in the journal so its first backup become acomplished.
	#
	# @param items The new paths that have been added to the watched ones
	# @author Adrián
	def __threadedBackupNewItems(self,items):
		#self.__bniLock.acquire()
		for path in items:
			if (not self.__snapshotManager.getVersionsInfo(path)[1]) and (not self.__journal.isPlanified(path) and not self.__checkLists(path)):
				self.__getContent(path, 'CREATE', path, path, 'initial')
				#self.__journal.insert(timestamp,'CREATE',path,path,'initial','0')
				#for item in completeContent:
				#	self.__journal.insert(timestamp,'CREATE',item,item,'initial','0')
		#self.__bniLock.release()
	
	## @brief Function to use with os.oath.walk to obtain the content of 
	# a subtreee of the directory tree. It will insert the required info into the journal
	#
	# @param dirname the current directory that we are visiting
	# @param filelist the content of the current directory
	# @param data contains :
	# 		 timestamp the timestamp of the event that causes the directory walk
	# 		 event contains the event name
	# 		 init contains the initial path of the event
	# 		 final the final path of the event
	# 		 requestor who request the operation
	#  
	#FIXME if we test the root path before calling the visitor we can avoid the double test done here
	#the first if will be unnecessary 
	def __visitor(self, data, dirname, filelist):
		if not self.__checkLists(dirname):
			#insert dirname into the journal
			# if init == final, the event raised is not a movement
			if data['init'] == data['final']:
				self.__journal.insert(data['timestamp'],data['event'],dirname,dirname,data['requestor'],'0')
			else:
				self.__journal.insert(data['timestamp'],data['event'],data['init'],dirname,data['requestor'],'0')
			#lets process the filelist
			for item in filelist:
				path = os.path.join(dirname,item)
				item = path
				if self.__checkLists(item):
					del item
				else:
					if not os.path.isdir(path):
						if data['init'] == data['final']:
							self.__journal.insert(data['timestamp'],data['event'],path,path,data['requestor'],'0')
						else:
							self.__journal.insert(data['timestamp'],data['event'],data['init'],path,data['requestor'],'0')
							
		#parent is blaclisted, all below is ignored
		else:
			del filelist
